#!/usr/bin/env python
#
# A library that provides a Python interface to the Telegram Bot API
# Copyright (C) 2015-2026
# Leandro Toledo de Souza <devs@python-telegram-bot.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser Public License for more details.
#
# You should have received a copy of the GNU Lesser Public License
# along with this program.  If not, see [http://www.gnu.org/licenses/].
"""This module contains the class that represent a Telegram InlineQueryResultsButton."""

from typing import TYPE_CHECKING, Final

from telegram import constants
from telegram._telegramobject import TelegramObject
from telegram._utils.argumentparsing import de_json_optional
from telegram._utils.types import JSONDict
from telegram._webappinfo import WebAppInfo

if TYPE_CHECKING:
    from telegram import Bot


class InlineQueryResultsButton(TelegramObject):
    """This object represents a button to be shown above inline query results. You **must** use
    exactly one of the optional fields.

    Objects of this class are comparable in terms of equality. Two objects of this class are
    considered equal, if their :attr:`text`, :attr:`web_app` and :attr:`start_parameter` are equal.

    Args:
        text (:obj:`str`): Label text on the button.
        web_app (:class:`telegram.WebAppInfo`, optional): Description of the
            `Web App <https://core.telegram.org/bots/webapps>`_ that will be launched when the
            user presses the button. The Web App will be able to switch back to the inline mode
            using the method
            `switchInlineQuery <https://core.telegram.org/bots/webapps#initializing-mini-apps>`_
            inside the Web App.
        start_parameter (:obj:`str`, optional):  Deep-linking parameter for the
            :guilabel:`/start` message sent to the bot when user presses the switch button.
            :tg-const:`telegram.constants.InlineQueryResultsButtonLimit.MIN_START_PARAMETER_LENGTH`
            -
            :tg-const:`telegram.constants.InlineQueryResultsButtonLimit.MAX_START_PARAMETER_LENGTH`
            characters, only ``A-Z``, ``a-z``, ``0-9``, ``_`` and ``-`` are allowed.

            Example:
                An inline bot that sends YouTube videos can ask the user to connect the bot to
                their YouTube account to adapt search results accordingly. To do this, it displays
                a 'Connect your YouTube account' button above the results, or even before showing
                any. The user presses the button, switches to a private chat with the bot and, in
                doing so, passes a start parameter that instructs the bot to return an OAuth link.
                Once done, the bot can offer a switch_inline button so that the user can easily
                return to the chat where they wanted to use the bot's inline capabilities.

    Attributes:
        text (:obj:`str`): Label text on the button.
        web_app (:class:`telegram.WebAppInfo`): Optional. Description of the
            `Web App <https://core.telegram.org/bots/webapps>`_ that will be launched when the
            user presses the button. The Web App will be able to switch back to the inline mode
            using the method ``web_app_switch_inline_query`` inside the Web App.
        start_parameter (:obj:`str`): Optional. Deep-linking parameter for the
            :tg-const:`telegram.constants.InlineQueryResultsButtonLimit.MIN_START_PARAMETER_LENGTH`
            -
            :tg-const:`telegram.constants.InlineQueryResultsButtonLimit.MAX_START_PARAMETER_LENGTH`
            characters, only ``A-Z``, ``a-z``, ``0-9``, ``_`` and ``-`` are allowed.

    """

    __slots__ = ("start_parameter", "text", "web_app")

    def __init__(
        self,
        text: str,
        web_app: WebAppInfo | None = None,
        start_parameter: str | None = None,
        *,
        api_kwargs: JSONDict | None = None,
    ):
        super().__init__(api_kwargs=api_kwargs)

        # Required
        self.text: str = text

        # Optional
        self.web_app: WebAppInfo | None = web_app
        self.start_parameter: str | None = start_parameter

        self._id_attrs = (self.text, self.web_app, self.start_parameter)

        self._freeze()

    @classmethod
    def de_json(cls, data: JSONDict, bot: "Bot | None" = None) -> "InlineQueryResultsButton":
        """See :meth:`telegram.TelegramObject.de_json`."""

        data["web_app"] = de_json_optional(data.get("web_app"), WebAppInfo, bot)

        return super().de_json(data=data, bot=bot)

    MIN_START_PARAMETER_LENGTH: Final[int] = (
        constants.InlineQueryResultsButtonLimit.MIN_START_PARAMETER_LENGTH
    )
    """:const:`telegram.constants.InlineQueryResultsButtonLimit.MIN_START_PARAMETER_LENGTH`"""
    MAX_START_PARAMETER_LENGTH: Final[int] = (
        constants.InlineQueryResultsButtonLimit.MAX_START_PARAMETER_LENGTH
    )
    """:const:`telegram.constants.InlineQueryResultsButtonLimit.MAX_START_PARAMETER_LENGTH`"""



#!/usr/bin/env python
#
# A library that provides a Python interface to the Telegram Bot API
# Copyright (C) 2015-2026
# Leandro Toledo de Souza <devs@python-telegram-bot.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser Public License for more details.
#
# You should have received a copy of the GNU Lesser Public License
# along with this program.  If not, see [http://www.gnu.org/licenses/].
"""This module contains the CallbackContext class."""

from collections.abc import Awaitable, Generator
from re import Match
from typing import TYPE_CHECKING, Any, Generic, NoReturn, TypeVar

from telegram._callbackquery import CallbackQuery
from telegram._update import Update
from telegram._utils.warnings import warn
from telegram.ext._extbot import ExtBot
from telegram.ext._utils.types import BD, BT, CD, UD

if TYPE_CHECKING:
    from asyncio import Future, Queue

    from telegram.ext import Application, Job, JobQueue
    from telegram.ext._utils.types import CCT

_STORING_DATA_WIKI = (
    "https://github.com/python-telegram-bot/python-telegram-bot"
    "/wiki/Storing-bot%2C-user-and-chat-related-data"
)

# something like poor mans "typing.Self" for py<3.11
ST = TypeVar("ST", bound="CallbackContext[Any, Any, Any, Any]")


class CallbackContext(Generic[BT, UD, CD, BD]):
    """
    This is a context object passed to the callback called by :class:`telegram.ext.BaseHandler`
    or by the :class:`telegram.ext.Application` in an error handler added by
    :attr:`telegram.ext.Application.add_error_handler` or to the callback of a
    :class:`telegram.ext.Job`.

    Note:
        :class:`telegram.ext.Application` will create a single context for an entire update. This
        means that if you got 2 handlers in different groups and they both get called, they will
        receive the same :class:`CallbackContext` object (of course with proper attributes like
        :attr:`matches` differing). This allows you to add custom attributes in a lower handler
        group callback, and then subsequently access those attributes in a higher handler group
        callback. Note that the attributes on :class:`CallbackContext` might change in the future,
        so make sure to use a fairly unique name for the attributes.

    Warning:
         Do not combine custom attributes with :paramref:`telegram.ext.BaseHandler.block` set to
         :obj:`False` or :attr:`telegram.ext.Application.concurrent_updates` set to
         :obj:`True`. Due to how those work, it will almost certainly execute the callbacks for an
         update out of order, and the attributes that you think you added will not be present.

    This class is a :class:`~typing.Generic` class and accepts four type variables:

    1. The type of :attr:`bot`. Must be :class:`telegram.Bot` or a subclass of that class.
    2. The type of :attr:`user_data` (if :attr:`user_data` is not :obj:`None`).
    3. The type of :attr:`chat_data` (if :attr:`chat_data` is not :obj:`None`).
    4. The type of :attr:`bot_data` (if :attr:`bot_data` is not :obj:`None`).

    Examples:
        * :any:`Context Types Bot <examples.contexttypesbot>`
        * :any:`Custom Webhook Bot <examples.customwebhookbot>`

    .. seealso:: :attr:`telegram.ext.ContextTypes.DEFAULT_TYPE`,
        :wiki:`Job Queue <Extensions---JobQueue>`

    Args:
        application (:class:`telegram.ext.Application`): The application associated with this
            context.
        chat_id (:obj:`int`, optional): The ID of the chat associated with this object. Used
            to provide :attr:`chat_data`.

            .. versionadded:: 20.0
        user_id (:obj:`int`, optional): The ID of the user associated with this object. Used
            to provide :attr:`user_data`.

            .. versionadded:: 20.0
    Attributes:
        coroutine (:term:`awaitable`): Optional. Only present in error handlers if the
            error was caused by an awaitable run with :meth:`Application.create_task` or a handler
            callback with :attr:`block=False <BaseHandler.block>`.
        matches (list[:meth:`re.Match <re.Match.expand>`]): Optional. If the associated update
            originated from a :class:`filters.Regex`, this will contain a list of match objects for
            every pattern where ``re.search(pattern, string)`` returned a match. Note that filters
            short circuit, so combined regex filters will not always be evaluated.
        args (list[:obj:`str`]): Optional. Arguments passed to a command if the associated update
            is handled by :class:`telegram.ext.CommandHandler`, :class:`telegram.ext.PrefixHandler`
            or :class:`telegram.ext.StringCommandHandler`. It contains a list of the words in the
            text after the command, using any whitespace string as a delimiter.
        error (:exc:`Exception`): Optional. The error that was raised. Only present when passed
            to an error handler registered with :attr:`telegram.ext.Application.add_error_handler`.
        job (:class:`telegram.ext.Job`): Optional. The job which originated this callback.
            Only present when passed to the callback of :class:`telegram.ext.Job` or in error
            handlers if the error is caused by a job.

            .. versionchanged:: 20.0
                :attr:`job` is now also present in error handlers if the error is caused by a job.

    """

    __slots__ = (
        "__dict__",
        "_application",
        "_chat_id",
        "_user_id",
        "args",
        "coroutine",
        "error",
        "job",
        "matches",
    )

    def __init__(
        self: ST,
        application: "Application[BT, ST, UD, CD, BD, Any]",
        chat_id: int | None = None,
        user_id: int | None = None,
    ):
        self._application: Application[BT, ST, UD, CD, BD, Any] = application
        self._chat_id: int | None = chat_id
        self._user_id: int | None = user_id
        self.args: list[str] | None = None
        self.matches: list[Match[str]] | None = None
        self.error: Exception | None = None
        self.job: Job[Any] | None = None
        self.coroutine: Generator[Future[object] | None, None, Any] | Awaitable[Any] | None = None

    @property
    def application(self) -> "Application[BT, ST, UD, CD, BD, Any]":
        """:class:`telegram.ext.Application`: The application associated with this context."""
        return self._application  # type: ignore[return-value]

    @property
    def bot_data(self) -> BD:
        """:obj:`ContextTypes.bot_data`: Optional. An object that can be used to keep any data in.
        For each update it will be the same :attr:`ContextTypes.bot_data`. Defaults to :obj:`dict`.

        .. seealso:: :wiki:`Storing Bot, User and Chat Related Data\
            <Storing-bot%2C-user-and-chat-related-data>`
        """
        return self.application.bot_data

    @bot_data.setter
    def bot_data(self, _: object) -> NoReturn:
        raise AttributeError(
            f"You can not assign a new value to bot_data, see {_STORING_DATA_WIKI}"
        )

    @property
    def chat_data(self) -> CD | None:
        """:obj:`ContextTypes.chat_data`: Optional. An object that can be used to keep any data in.
        For each update from the same chat id it will be the same :obj:`ContextTypes.chat_data`.
        Defaults to :obj:`dict`.

        Warning:
            When a group chat migrates to a supergroup, its chat id will change and the
            ``chat_data`` needs to be transferred. For details see our
            :wiki:`wiki page <Storing-bot,-user-and-chat-related-data#chat-migration>`.

        .. seealso:: :wiki:`Storing Bot, User and Chat Related Data\
            <Storing-bot%2C-user-and-chat-related-data>`

        .. versionchanged:: 20.0
            The chat data is now also present in error handlers if the error is caused by a job.
        """
        if self._chat_id is not None:
            return self._application.chat_data[self._chat_id]
        return None

    @chat_data.setter
    def chat_data(self, _: object) -> NoReturn:
        raise AttributeError(
            f"You can not assign a new value to chat_data, see {_STORING_DATA_WIKI}"
        )

    @property
    def user_data(self) -> UD | None:
        """:obj:`ContextTypes.user_data`: Optional. An object that can be used to keep any data in.
        For each update from the same user it will be the same :obj:`ContextTypes.user_data`.
        Defaults to :obj:`dict`.

        .. seealso:: :wiki:`Storing Bot, User and Chat Related Data\
            <Storing-bot%2C-user-and-chat-related-data>`

        .. versionchanged:: 20.0
            The user data is now also present in error handlers if the error is caused by a job.
        """
        if self._user_id is not None:
            return self._application.user_data[self._user_id]
        return None

    @user_data.setter
    def user_data(self, _: object) -> NoReturn:
        raise AttributeError(
            f"You can not assign a new value to user_data, see {_STORING_DATA_WIKI}"
        )

    async def refresh_data(self) -> None:
        """If :attr:`application` uses persistence, calls
        :meth:`telegram.ext.BasePersistence.refresh_bot_data` on :attr:`bot_data`,
        :meth:`telegram.ext.BasePersistence.refresh_chat_data` on :attr:`chat_data` and
        :meth:`telegram.ext.BasePersistence.refresh_user_data` on :attr:`user_data`, if
        appropriate.

        Will be called by :meth:`telegram.ext.Application.process_update` and
        :meth:`telegram.ext.Job.run`.

        .. versionadded:: 13.6
        """
        if self.application.persistence:
            if self.application.persistence.store_data.bot_data:
                await self.application.persistence.refresh_bot_data(self.bot_data)
            if self.application.persistence.store_data.chat_data and self._chat_id is not None:
                await self.application.persistence.refresh_chat_data(
                    chat_id=self._chat_id,
                    chat_data=self.chat_data,  # type: ignore[arg-type]
                )
            if self.application.persistence.store_data.user_data and self._user_id is not None:
                await self.application.persistence.refresh_user_data(
                    user_id=self._user_id,
                    user_data=self.user_data,  # type: ignore[arg-type]
                )

    def drop_callback_data(self, callback_query: CallbackQuery) -> None:
        """
        Deletes the cached data for the specified callback query.

        .. versionadded:: 13.6

        Note:
            Will *not* raise exceptions in case the data is not found in the cache.
            *Will* raise :exc:`KeyError` in case the callback query can not be found in the cache.

        .. seealso:: :wiki:`Arbitrary callback_data <Arbitrary-callback_data>`

        Args:
            callback_query (:class:`telegram.CallbackQuery`): The callback query.

        Raises:
            KeyError | RuntimeError: :exc:`KeyError`, if the callback query can not be found in
                the cache and :exc:`RuntimeError`, if the bot doesn't allow for arbitrary
                callback data.
        """
        if isinstance(self.bot, ExtBot):
            if self.bot.callback_data_cache is None:
                raise RuntimeError(
                    "This telegram.ext.ExtBot instance does not use arbitrary callback data."
                )
            self.bot.callback_data_cache.drop_data(callback_query)
        else:
            raise RuntimeError(  # noqa: TRY004
                "telegram.Bot does not allow for arbitrary callback data."
            )

    @classmethod
    def from_error(
        cls: type["CCT"],
        update: object,
        error: Exception,
        application: "Application[BT, CCT, UD, CD, BD, Any]",
        job: "Job[Any] | None" = None,
        coroutine: Generator["Future[object] | None", None, Any] | Awaitable[Any] | None = None,
    ) -> "CCT":
        """
        Constructs an instance of :class:`telegram.ext.CallbackContext` to be passed to the error
        handlers.

        .. seealso:: :meth:`telegram.ext.Application.add_error_handler`

        .. versionchanged:: 20.0
            Removed arguments ``async_args`` and ``async_kwargs``.

        Args:
            update (:obj:`object` | :class:`telegram.Update`): The update associated with the
                error. May be :obj:`None`, e.g. for errors in job callbacks.
            error (:obj:`Exception`): The error.
            application (:class:`telegram.ext.Application`): The application associated with this
                context.
            job (:class:`telegram.ext.Job`, optional): The job associated with the error.

                .. versionadded:: 20.0
            coroutine (:term:`awaitable`, optional): The awaitable associated
                with this error if the error was caused by a coroutine run with
                :meth:`Application.create_task` or a handler callback with
                :attr:`block=False <BaseHandler.block>`.

                .. versionadded:: 20.0

                .. versionchanged:: 20.2
                    Accepts :class:`asyncio.Future` and generator-based coroutine functions.
        Returns:
            :class:`telegram.ext.CallbackContext`
        """
        # update and job will never be present at the same time
        if update is not None:
            self = cls.from_update(update, application)
        elif job is not None:
            self = cls.from_job(job, application)
        else:
            self = cls(application)  # type: ignore

        self.error = error
        self.coroutine = coroutine
        return self

    @classmethod
    def from_update(
        cls: type["CCT"],
        update: object,
        application: "Application[Any, CCT, Any, Any, Any, Any]",
    ) -> "CCT":
        """
        Constructs an instance of :class:`telegram.ext.CallbackContext` to be passed to the
        handlers.

        .. seealso:: :meth:`telegram.ext.Application.add_handler`

        Args:
            update (:obj:`object` | :class:`telegram.Update`): The update.
            application (:class:`telegram.ext.Application`): The application associated with this
                context.

        Returns:
            :class:`telegram.ext.CallbackContext`
        """
        if isinstance(update, Update):
            chat = update.effective_chat
            user = update.effective_user

            chat_id = chat.id if chat else None
            user_id = user.id if user else None

            return cls(application, chat_id=chat_id, user_id=user_id)  # type: ignore
        return cls(application)  # type: ignore

    @classmethod
    def from_job(
        cls: type["CCT"],
        job: "Job[CCT]",
        application: "Application[Any, CCT, Any, Any, Any, Any]",
    ) -> "CCT":
        """
        Constructs an instance of :class:`telegram.ext.CallbackContext` to be passed to a
        job callback.

        .. seealso:: :meth:`telegram.ext.JobQueue`

        Args:
            job (:class:`telegram.ext.Job`): The job.
            application (:class:`telegram.ext.Application`): The application associated with this
                context.

        Returns:
            :class:`telegram.ext.CallbackContext`
        """
        self = cls(application, chat_id=job.chat_id, user_id=job.user_id)  # type: ignore
        self.job = job
        return self

    def update(self, data: dict[str, object]) -> None:
        """Updates ``self.__slots__`` with the passed data.

        Args:
            data (dict[:obj:`str`, :obj:`object`]): The data.
        """
        for key, value in data.items():
            setattr(self, key, value)

    @property
    def bot(self) -> BT:
        """:class:`telegram.Bot`: The bot associated with this context."""
        return self._application.bot

    @property
    def job_queue(self) -> "JobQueue[ST] | None":
        """
        :class:`telegram.ext.JobQueue`: The :class:`JobQueue` used by the
        :class:`telegram.ext.Application`.

        .. seealso:: :wiki:`Job Queue <Extensions---JobQueue>`
        """
        if self._application._job_queue is None:  # pylint: disable=protected-access
            warn(
                "No `JobQueue` set up. To use `JobQueue`, you must install PTB via "
                '`pip install "python-telegram-bot[job-queue]"`.',
                stacklevel=2,
            )
        return self._application._job_queue  # pylint: disable=protected-access

    @property
    def update_queue(self) -> "Queue[object]":
        """
        :class:`asyncio.Queue`: The :class:`asyncio.Queue` instance used by the
            :class:`telegram.ext.Application` and (usually) the :class:`telegram.ext.Updater`
            associated with this context.

        """
        return self._application.update_queue

    @property
    def match(self) -> Match[str] | None:
        """
        :meth:`re.Match <re.Match.expand>`: The first match from :attr:`matches`.
            Useful if you are only filtering using a single regex filter.
            Returns :obj:`None` if :attr:`matches` is empty.
        """
        try:
            return self.matches[0]  # type: ignore[index] # pylint: disable=unsubscriptable-object
        except (IndexError, TypeError):
            return None



#!/usr/bin/env python
#
# A library that provides a Python interface to the Telegram Bot API
# Copyright (C) 2015-2026
# Leandro Toledo de Souza <devs@python-telegram-bot.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser Public License for more details.
#
# You should have received a copy of the GNU Lesser Public License
# along with this program.  If not, see [http://www.gnu.org/licenses/].

import pytest

from telegram import EncryptedCredentials, PassportElementError
from tests.auxil.slots import mro_slots


@pytest.fixture(scope="module")
def encrypted_credentials():
    return EncryptedCredentials(
        EncryptedCredentialsTestBase.data,
        EncryptedCredentialsTestBase.hash,
        EncryptedCredentialsTestBase.secret,
    )


class EncryptedCredentialsTestBase:
    data = "data"
    hash = "hash"
    secret = "secret"


class TestEncryptedCredentialsWithoutRequest(EncryptedCredentialsTestBase):
    def test_slot_behaviour(self, encrypted_credentials):
        inst = encrypted_credentials
        for attr in inst.__slots__:
            assert getattr(inst, attr, "err") != "err", f"got extra slot '{attr}'"
        assert len(mro_slots(inst)) == len(set(mro_slots(inst))), "duplicate slot"

    def test_expected_values(self, encrypted_credentials):
        assert encrypted_credentials.data == self.data
        assert encrypted_credentials.hash == self.hash
        assert encrypted_credentials.secret == self.secret

    def test_to_dict(self, encrypted_credentials):
        encrypted_credentials_dict = encrypted_credentials.to_dict()

        assert isinstance(encrypted_credentials_dict, dict)
        assert encrypted_credentials_dict["data"] == encrypted_credentials.data
        assert encrypted_credentials_dict["hash"] == encrypted_credentials.hash
        assert encrypted_credentials_dict["secret"] == encrypted_credentials.secret

    def test_equality(self):
        a = EncryptedCredentials(self.data, self.hash, self.secret)
        b = EncryptedCredentials(self.data, self.hash, self.secret)
        c = EncryptedCredentials(self.data, "", "")
        d = EncryptedCredentials("", self.hash, "")
        e = EncryptedCredentials("", "", self.secret)
        f = PassportElementError("source", "type", "message")

        assert a == b
        assert hash(a) == hash(b)
        assert a is not b

        assert a != c
        assert hash(a) != hash(c)

        assert a != d
        assert hash(a) != hash(d)

        assert a != e
        assert hash(a) != hash(e)

        assert a != f
        assert hash(a) != hash(f)



#!/usr/bin/env python
#
# A library that provides a Python interface to the Telegram Bot API
# Copyright (C) 2015-2026
# Leandro Toledo de Souza <devs@python-telegram-bot.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser Public License for more details.
#
# You should have received a copy of the GNU Lesser Public License
# along with this program.  If not, see [http://www.gnu.org/licenses/].
import datetime as dtm

import pytest

from telegram import Bot, File, PassportElementError, PassportFile
from telegram._utils.datetime import UTC, to_timestamp
from tests.auxil.bot_method_checks import (
    check_defaults_handling,
    check_shortcut_call,
    check_shortcut_signature,
)
from tests.auxil.slots import mro_slots


@pytest.fixture(scope="class")
def passport_file(bot):
    pf = PassportFile(
        file_id=PassportFileTestBase.file_id,
        file_unique_id=PassportFileTestBase.file_unique_id,
        file_size=PassportFileTestBase.file_size,
        file_date=PassportFileTestBase.file_date,
    )
    pf.set_bot(bot)
    return pf


class PassportFileTestBase:
    file_id = "data"
    file_unique_id = "adc3145fd2e84d95b64d68eaa22aa33e"
    file_size = 50
    file_date = dtm.datetime.now(tz=UTC).replace(microsecond=0)


class TestPassportFileWithoutRequest(PassportFileTestBase):
    def test_slot_behaviour(self, passport_file):
        inst = passport_file
        for attr in inst.__slots__:
            assert getattr(inst, attr, "err") != "err", f"got extra slot '{attr}'"
        assert len(mro_slots(inst)) == len(set(mro_slots(inst))), "duplicate slot"

    def test_expected_values(self, passport_file):
        assert passport_file.file_id == self.file_id
        assert passport_file.file_unique_id == self.file_unique_id
        assert passport_file.file_size == self.file_size
        assert passport_file.file_date == self.file_date

    def test_to_dict(self, passport_file):
        passport_file_dict = passport_file.to_dict()

        assert isinstance(passport_file_dict, dict)
        assert passport_file_dict["file_id"] == passport_file.file_id
        assert passport_file_dict["file_unique_id"] == passport_file.file_unique_id
        assert passport_file_dict["file_size"] == passport_file.file_size
        assert passport_file_dict["file_date"] == to_timestamp(passport_file.file_date)

    def test_de_json_localization(self, passport_file, tz_bot, offline_bot, raw_bot):
        json_dict = {
            "file_id": self.file_id,
            "file_unique_id": self.file_unique_id,
            "file_size": self.file_size,
            "file_date": to_timestamp(self.file_date),
        }

        pf = PassportFile.de_json(json_dict, offline_bot)
        pf_raw = PassportFile.de_json(json_dict, raw_bot)
        pf_tz = PassportFile.de_json(json_dict, tz_bot)

        # comparing utcoffsets because comparing timezones is unpredicatable
        date_offset = pf_tz.file_date.utcoffset()
        tz_bot_offset = tz_bot.defaults.tzinfo.utcoffset(pf_tz.file_date.replace(tzinfo=None))

        assert pf_raw.file_date.tzinfo == UTC
        assert pf.file_date.tzinfo == UTC
        assert date_offset == tz_bot_offset

    def test_equality(self):
        a = PassportFile(self.file_id, self.file_unique_id, self.file_size, self.file_date)
        b = PassportFile("", self.file_unique_id, self.file_size, self.file_date)
        c = PassportFile(self.file_id, self.file_unique_id, "", "")
        d = PassportFile("", "", self.file_size, self.file_date)
        e = PassportElementError("source", "type", "message")

        assert a == b
        assert hash(a) == hash(b)
        assert a is not b

        assert a == c
        assert hash(a) == hash(c)

        assert a != d
        assert hash(a) != hash(d)

        assert a != e
        assert hash(a) != hash(e)

    async def test_get_file_instance_method(self, monkeypatch, passport_file):
        async def make_assertion(*_, **kwargs):
            result = kwargs["file_id"] == passport_file.file_id
            # we need to be a bit hacky here, b/c PF.get_file needs Bot.get_file to return a File
            return File(file_id=result, file_unique_id=result)

        assert check_shortcut_signature(PassportFile.get_file, Bot.get_file, ["file_id"], [])
        assert await check_shortcut_call(
            passport_file.get_file, passport_file.get_bot(), "get_file"
        )
        assert await check_defaults_handling(passport_file.get_file, passport_file.get_bot())

        monkeypatch.setattr(passport_file.get_bot(), "get_file", make_assertion)
        assert (await passport_file.get_file()).file_id == "True"



#!/usr/bin/env python
#
# A library that provides a Python interface to the Telegram Bot API
# Copyright (C) 2015-2026
# Leandro Toledo de Souza <devs@python-telegram-bot.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser Public License for more details.
#
# You should have received a copy of the GNU Lesser Public License
# along with this program.  If not, see [http://www.gnu.org/licenses/].
import datetime as dtm

import pytest

from telegram import (
    InlineKeyboardButton,
    InlineKeyboardMarkup,
    InlineQueryResultAudio,
    InlineQueryResultVoice,
    InputTextMessageContent,
    MessageEntity,
)
from telegram.warnings import PTBDeprecationWarning
from tests.auxil.slots import mro_slots


@pytest.fixture(scope="module")
def inline_query_result_voice():
    return InlineQueryResultVoice(
        id=InlineQueryResultVoiceTestBase.id_,
        voice_url=InlineQueryResultVoiceTestBase.voice_url,
        title=InlineQueryResultVoiceTestBase.title,
        voice_duration=InlineQueryResultVoiceTestBase.voice_duration,
        caption=InlineQueryResultVoiceTestBase.caption,
        parse_mode=InlineQueryResultVoiceTestBase.parse_mode,
        caption_entities=InlineQueryResultVoiceTestBase.caption_entities,
        input_message_content=InlineQueryResultVoiceTestBase.input_message_content,
        reply_markup=InlineQueryResultVoiceTestBase.reply_markup,
    )


class InlineQueryResultVoiceTestBase:
    id_ = "id"
    type_ = "voice"
    voice_url = "voice url"
    title = "title"
    voice_duration = dtm.timedelta(seconds=10)
    caption = "caption"
    parse_mode = "HTML"
    caption_entities = [MessageEntity(MessageEntity.ITALIC, 0, 7)]
    input_message_content = InputTextMessageContent("input_message_content")
    reply_markup = InlineKeyboardMarkup([[InlineKeyboardButton("reply_markup")]])


class TestInlineQueryResultVoiceWithoutRequest(InlineQueryResultVoiceTestBase):
    def test_slot_behaviour(self, inline_query_result_voice):
        inst = inline_query_result_voice
        for attr in inst.__slots__:
            assert getattr(inst, attr, "err") != "err", f"got extra slot '{attr}'"
        assert len(mro_slots(inst)) == len(set(mro_slots(inst))), "duplicate slot"

    def test_expected_values(self, inline_query_result_voice):
        assert inline_query_result_voice.type == self.type_
        assert inline_query_result_voice.id == self.id_
        assert inline_query_result_voice.voice_url == self.voice_url
        assert inline_query_result_voice.title == self.title
        assert inline_query_result_voice._voice_duration == self.voice_duration
        assert inline_query_result_voice.caption == self.caption
        assert inline_query_result_voice.parse_mode == self.parse_mode
        assert inline_query_result_voice.caption_entities == tuple(self.caption_entities)
        assert (
            inline_query_result_voice.input_message_content.to_dict()
            == self.input_message_content.to_dict()
        )
        assert inline_query_result_voice.reply_markup.to_dict() == self.reply_markup.to_dict()

    def test_caption_entities_always_tuple(self):
        result = InlineQueryResultVoice(
            self.id_,
            self.voice_url,
            self.title,
        )

        assert result.caption_entities == ()

    def test_to_dict(self, inline_query_result_voice):
        inline_query_result_voice_dict = inline_query_result_voice.to_dict()

        assert isinstance(inline_query_result_voice_dict, dict)
        assert inline_query_result_voice_dict["type"] == inline_query_result_voice.type
        assert inline_query_result_voice_dict["id"] == inline_query_result_voice.id
        assert inline_query_result_voice_dict["voice_url"] == inline_query_result_voice.voice_url
        assert inline_query_result_voice_dict["title"] == inline_query_result_voice.title
        assert inline_query_result_voice_dict["voice_duration"] == int(
            self.voice_duration.total_seconds()
        )
        assert isinstance(inline_query_result_voice_dict["voice_duration"], int)
        assert inline_query_result_voice_dict["caption"] == inline_query_result_voice.caption
        assert inline_query_result_voice_dict["parse_mode"] == inline_query_result_voice.parse_mode
        assert inline_query_result_voice_dict["caption_entities"] == [
            ce.to_dict() for ce in inline_query_result_voice.caption_entities
        ]
        assert (
            inline_query_result_voice_dict["input_message_content"]
            == inline_query_result_voice.input_message_content.to_dict()
        )
        assert (
            inline_query_result_voice_dict["reply_markup"]
            == inline_query_result_voice.reply_markup.to_dict()
        )

    def test_time_period_properties(self, PTB_TIMEDELTA, inline_query_result_voice):
        voice_duration = inline_query_result_voice.voice_duration

        if PTB_TIMEDELTA:
            assert voice_duration == self.voice_duration
            assert isinstance(voice_duration, dtm.timedelta)
        else:
            assert voice_duration == int(self.voice_duration.total_seconds())
            assert isinstance(voice_duration, int)

    def test_time_period_int_deprecated(self, recwarn, PTB_TIMEDELTA, inline_query_result_voice):
        inline_query_result_voice.voice_duration

        if PTB_TIMEDELTA:
            assert len(recwarn) == 0
        else:
            assert len(recwarn) == 1
            assert "`voice_duration` will be of type `datetime.timedelta`" in str(
                recwarn[0].message
            )
            assert recwarn[0].category is PTBDeprecationWarning

    def test_equality(self):
        a = InlineQueryResultVoice(self.id_, self.voice_url, self.title)
        b = InlineQueryResultVoice(self.id_, self.voice_url, self.title)
        c = InlineQueryResultVoice(self.id_, "", self.title)
        d = InlineQueryResultVoice("", self.voice_url, self.title)
        e = InlineQueryResultAudio(self.id_, "", "")

        assert a == b
        assert hash(a) == hash(b)
        assert a is not b

        assert a == c
        assert hash(a) == hash(c)

        assert a != d
        assert hash(a) != hash(d)

        assert a != e
        assert hash(a) != hash(e)



#!/usr/bin/env python
#
# A library that provides a Python interface to the Telegram Bot API
# Copyright (C) 2015-2026
# Leandro Toledo de Souza <devs@python-telegram-bot.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser Public License for more details.
#
# You should have received a copy of the GNU Lesser Public License
# along with this program.  If not, see [http://www.gnu.org/licenses/].
"""This module contains an objects related to Telegram checklists."""

import datetime as dtm
from collections.abc import Sequence
from typing import TYPE_CHECKING, Optional

from telegram._chat import Chat
from telegram._messageentity import MessageEntity
from telegram._telegramobject import TelegramObject
from telegram._user import User
from telegram._utils.argumentparsing import de_json_optional, de_list_optional, parse_sequence_arg
from telegram._utils.datetime import extract_tzinfo_from_defaults, from_timestamp
from telegram._utils.entities import parse_message_entities, parse_message_entity
from telegram._utils.types import JSONDict
from telegram.constants import ZERO_DATE

if TYPE_CHECKING:
    from telegram import Bot, Message


class ChecklistTask(TelegramObject):
    """
    Describes a task in a checklist.

    Objects of this class are comparable in terms of equality.
    Two objects of this class are considered equal, if their :attr:`id` is equal.

    .. versionadded:: 22.3

    Args:
        id (:obj:`int`): Unique identifier of the task.
        text (:obj:`str`): Text of the task.
        text_entities (Sequence[:class:`telegram.MessageEntity`], optional): Special
            entities that appear in the task text.
        completed_by_user (:class:`telegram.User`, optional): User that completed the task; omitted
            if the task wasn't completed
        completed_by_chat (:class:`telegram.Chat`, optional): Chat that completed the task; omitted
            if the task wasn't completed by a chat

            .. versionadded:: 22.6
        completion_date (:class:`datetime.datetime`, optional): Point in time when
            the task was completed; :attr:`~telegram.constants.ZERO_DATE` if the task wasn't
            completed

            |datetime_localization|

    Attributes:
        id (:obj:`int`): Unique identifier of the task.
        text (:obj:`str`): Text of the task.
        text_entities (Tuple[:class:`telegram.MessageEntity`]): Optional. Special
            entities that appear in the task text.
        completed_by_user (:class:`telegram.User`): Optional. User that completed the task; omitted
            if the task wasn't completed
        completed_by_chat (:class:`telegram.Chat`): Optional. Chat that completed the task; omitted
            if the task wasn't completed by a chat

            .. versionadded:: 22.6
        completion_date (:class:`datetime.datetime`): Optional. Point in time when
            the task was completed; :attr:`~telegram.constants.ZERO_DATE` if the task wasn't
            completed

            |datetime_localization|
    """

    __slots__ = (
        "completed_by_chat",
        "completed_by_user",
        "completion_date",
        "id",
        "text",
        "text_entities",
    )

    def __init__(
        self,
        id: int,  # pylint: disable=redefined-builtin
        text: str,
        text_entities: Sequence[MessageEntity] | None = None,
        completed_by_user: User | None = None,
        completion_date: dtm.datetime | None = None,
        completed_by_chat: Chat | None = None,
        *,
        api_kwargs: JSONDict | None = None,
    ):
        super().__init__(api_kwargs=api_kwargs)
        self.id: int = id
        self.text: str = text
        self.text_entities: tuple[MessageEntity, ...] = parse_sequence_arg(text_entities)
        self.completed_by_user: User | None = completed_by_user
        self.completed_by_chat: Chat | None = completed_by_chat
        self.completion_date: dtm.datetime | None = completion_date

        self._id_attrs = (self.id,)

        self._freeze()

    @classmethod
    def de_json(cls, data: JSONDict, bot: Optional["Bot"] = None) -> "ChecklistTask":
        """See :meth:`telegram.TelegramObject.de_json`."""
        data = cls._parse_data(data)

        # Get the local timezone from the bot if it has defaults
        loc_tzinfo = extract_tzinfo_from_defaults(bot)

        if (date := data.get("completion_date")) == 0:
            data["completion_date"] = ZERO_DATE
        else:
            data["completion_date"] = from_timestamp(date, tzinfo=loc_tzinfo)

        data["completed_by_user"] = de_json_optional(data.get("completed_by_user"), User, bot)
        data["completed_by_chat"] = de_json_optional(data.get("completed_by_chat"), Chat, bot)
        data["text_entities"] = de_list_optional(data.get("text_entities"), MessageEntity, bot)

        return super().de_json(data=data, bot=bot)

    def parse_entity(self, entity: MessageEntity) -> str:
        """Returns the text in :attr:`text`
        from a given :class:`telegram.MessageEntity` of :attr:`text_entities`.

        Note:
            This method is present because Telegram calculates the offset and length in
            UTF-16 codepoint pairs, which some versions of Python don't handle automatically.
            (That is, you can't just slice ``ChecklistTask.text`` with the offset and length.)

        Args:
            entity (:class:`telegram.MessageEntity`): The entity to extract the text from. It must
                be an entity that belongs to :attr:`text_entities`.

        Returns:
            :obj:`str`: The text of the given entity.
        """
        return parse_message_entity(self.text, entity)

    def parse_entities(self, types: list[str] | None = None) -> dict[MessageEntity, str]:
        """
        Returns a :obj:`dict` that maps :class:`telegram.MessageEntity` to :obj:`str`.
        It contains entities from this checklist task filtered by their ``type`` attribute as
        the key, and the text that each entity belongs to as the value of the :obj:`dict`.

        Note:
            This method should always be used instead of the :attr:`text_entities`
            attribute, since it calculates the correct substring from the message text based on
            UTF-16 codepoints. See :attr:`parse_entity` for more info.

        Args:
            types (list[:obj:`str`], optional): List of ``MessageEntity`` types as strings. If the
                    ``type`` attribute of an entity is contained in this list, it will be returned.
                    Defaults to :attr:`telegram.MessageEntity.ALL_TYPES`.

        Returns:
            dict[:class:`telegram.MessageEntity`, :obj:`str`]: A dictionary of entities mapped to
            the text that belongs to them, calculated based on UTF-16 codepoints.
        """
        return parse_message_entities(self.text, self.text_entities, types)


class Checklist(TelegramObject):
    """
    Describes a checklist.

    Objects of this class are comparable in terms of equality.
    Two objects of this class are considered equal, if all their :attr:`tasks` are equal.

    .. versionadded:: 22.3

    Args:
        title (:obj:`str`): Title of the checklist.
        title_entities (Sequence[:class:`telegram.MessageEntity`], optional): Special
            entities that appear in the checklist title.
        tasks (Sequence[:class:`telegram.ChecklistTask`]): List of tasks in the checklist.
        others_can_add_tasks (:obj:`bool`, optional): :obj:`True` if users other than the creator
            of the list can add tasks to the list
        others_can_mark_tasks_as_done (:obj:`bool`, optional): :obj:`True` if users other than the
            creator of the list can mark tasks as done or not done

    Attributes:
        title (:obj:`str`): Title of the checklist.
        title_entities (Tuple[:class:`telegram.MessageEntity`]): Optional. Special
            entities that appear in the checklist title.
        tasks (Tuple[:class:`telegram.ChecklistTask`]): List of tasks in the checklist.
        others_can_add_tasks (:obj:`bool`): Optional. :obj:`True` if users other than the creator
            of the list can add tasks to the list
        others_can_mark_tasks_as_done (:obj:`bool`): Optional. :obj:`True` if users other than the
            creator of the list can mark tasks as done or not done
    """

    __slots__ = (
        "others_can_add_tasks",
        "others_can_mark_tasks_as_done",
        "tasks",
        "title",
        "title_entities",
    )

    def __init__(
        self,
        title: str,
        tasks: Sequence[ChecklistTask],
        title_entities: Sequence[MessageEntity] | None = None,
        others_can_add_tasks: bool | None = None,
        others_can_mark_tasks_as_done: bool | None = None,
        *,
        api_kwargs: JSONDict | None = None,
    ):
        super().__init__(api_kwargs=api_kwargs)
        self.title: str = title
        self.title_entities: tuple[MessageEntity, ...] = parse_sequence_arg(title_entities)
        self.tasks: tuple[ChecklistTask, ...] = parse_sequence_arg(tasks)
        self.others_can_add_tasks: bool | None = others_can_add_tasks
        self.others_can_mark_tasks_as_done: bool | None = others_can_mark_tasks_as_done

        self._id_attrs = (self.tasks,)

        self._freeze()

    @classmethod
    def de_json(cls, data: JSONDict, bot: Optional["Bot"] = None) -> "Checklist":
        """See :meth:`telegram.TelegramObject.de_json`."""
        data = cls._parse_data(data)

        data["title_entities"] = de_list_optional(data.get("title_entities"), MessageEntity, bot)
        data["tasks"] = de_list_optional(data.get("tasks"), ChecklistTask, bot)

        return super().de_json(data=data, bot=bot)

    def parse_entity(self, entity: MessageEntity) -> str:
        """Returns the text in :attr:`title`
        from a given :class:`telegram.MessageEntity` of :attr:`title_entities`.

        Note:
            This method is present because Telegram calculates the offset and length in
            UTF-16 codepoint pairs, which some versions of Python don't handle automatically.
            (That is, you can't just slice :attr:`title` with the offset and length.)

        Args:
            entity (:class:`telegram.MessageEntity`): The entity to extract the text from. It must
                be an entity that belongs to :attr:`title_entities`.

        Returns:
            :obj:`str`: The text of the given entity.
        """
        return parse_message_entity(self.title, entity)

    def parse_entities(self, types: list[str] | None = None) -> dict[MessageEntity, str]:
        """
        Returns a :obj:`dict` that maps :class:`telegram.MessageEntity` to :obj:`str`.
        It contains entities from this checklist's title filtered by their ``type`` attribute as
        the key, and the text that each entity belongs to as the value of the :obj:`dict`.

        Note:
            This method should always be used instead of the :attr:`title_entities`
            attribute, since it calculates the correct substring from the message text based on
            UTF-16 codepoints. See :attr:`parse_entity` for more info.

        Args:
            types (list[:obj:`str`], optional): List of ``MessageEntity`` types as strings. If the
                    ``type`` attribute of an entity is contained in this list, it will be returned.
                    Defaults to :attr:`telegram.MessageEntity.ALL_TYPES`.

        Returns:
            dict[:class:`telegram.MessageEntity`, :obj:`str`]: A dictionary of entities mapped to
            the text that belongs to them, calculated based on UTF-16 codepoints.
        """
        return parse_message_entities(self.title, self.title_entities, types)


class ChecklistTasksDone(TelegramObject):
    """
    Describes a service message about checklist tasks marked as done or not done.

    Objects of this class are comparable in terms of equality.
    Two objects of this class are considered equal, if their :attr:`marked_as_done_task_ids` and
    :attr:`marked_as_not_done_task_ids` are equal.

    .. versionadded:: 22.3

    Args:
        checklist_message (:class:`telegram.Message`, optional): Message containing the checklist
            whose tasks were marked as done or not done. Note that the ~:class:`telegram.Message`
            object in this field will not contain the :attr:`~telegram.Message.reply_to_message`
            field even if it itself is a reply.
        marked_as_done_task_ids (Sequence[:obj:`int`], optional): Identifiers of the tasks that
            were marked as done
        marked_as_not_done_task_ids (Sequence[:obj:`int`], optional): Identifiers of the tasks that
            were marked as not done

    Attributes:
        checklist_message (:class:`telegram.Message`): Optional. Message containing the checklist
            whose tasks were marked as done or not done. Note that the ~:class:`telegram.Message`
            object in this field will not contain the :attr:`~telegram.Message.reply_to_message`
            field even if it itself is a reply.
        marked_as_done_task_ids (Tuple[:obj:`int`]): Optional. Identifiers of the tasks that were
            marked as done
        marked_as_not_done_task_ids (Tuple[:obj:`int`]): Optional. Identifiers of the tasks that
            were marked as not done
    """

    __slots__ = (
        "checklist_message",
        "marked_as_done_task_ids",
        "marked_as_not_done_task_ids",
    )

    def __init__(
        self,
        checklist_message: Optional["Message"] = None,
        marked_as_done_task_ids: Sequence[int] | None = None,
        marked_as_not_done_task_ids: Sequence[int] | None = None,
        *,
        api_kwargs: JSONDict | None = None,
    ):
        super().__init__(api_kwargs=api_kwargs)
        self.checklist_message: Message | None = checklist_message
        self.marked_as_done_task_ids: tuple[int, ...] = parse_sequence_arg(marked_as_done_task_ids)
        self.marked_as_not_done_task_ids: tuple[int, ...] = parse_sequence_arg(
            marked_as_not_done_task_ids
        )

        self._id_attrs = (self.marked_as_done_task_ids, self.marked_as_not_done_task_ids)

        self._freeze()

    @classmethod
    def de_json(cls, data: JSONDict, bot: Optional["Bot"] = None) -> "ChecklistTasksDone":
        """See :meth:`telegram.TelegramObject.de_json`."""
        data = cls._parse_data(data)

        # needs to be imported here to avoid circular import issues
        from telegram import Message  # pylint: disable=import-outside-toplevel  # noqa: PLC0415

        data["checklist_message"] = de_json_optional(data.get("checklist_message"), Message, bot)

        return super().de_json(data=data, bot=bot)


class ChecklistTasksAdded(TelegramObject):
    """
    Describes a service message about tasks added to a checklist.

    Objects of this class are comparable in terms of equality.
    Two objects of this class are considered equal, if their :attr:`tasks` are equal.

    .. versionadded:: 22.3

    Args:
        checklist_message (:class:`telegram.Message`, optional): Message containing the checklist
            to which tasks were added. Note that the ~:class:`telegram.Message`
            object in this field will not contain the :attr:`~telegram.Message.reply_to_message`
            field even if it itself is a reply.
        tasks (Sequence[:class:`telegram.ChecklistTask`]): List of tasks added to the checklist

    Attributes:
        checklist_message (:class:`telegram.Message`): Optional. Message containing the checklist
            to which tasks were added. Note that the ~:class:`telegram.Message`
            object in this field will not contain the :attr:`~telegram.Message.reply_to_message`
            field even if it itself is a reply.
        tasks (Tuple[:class:`telegram.ChecklistTask`]): List of tasks added to the checklist
    """

    __slots__ = ("checklist_message", "tasks")

    def __init__(
        self,
        tasks: Sequence[ChecklistTask],
        checklist_message: Optional["Message"] = None,
        *,
        api_kwargs: JSONDict | None = None,
    ):
        super().__init__(api_kwargs=api_kwargs)
        self.checklist_message: Message | None = checklist_message
        self.tasks: tuple[ChecklistTask, ...] = parse_sequence_arg(tasks)

        self._id_attrs = (self.tasks,)

        self._freeze()

    @classmethod
    def de_json(cls, data: JSONDict, bot: Optional["Bot"] = None) -> "ChecklistTasksAdded":
        """See :meth:`telegram.TelegramObject.de_json`."""
        data = cls._parse_data(data)

        # needs to be imported here to avoid circular import issues
        from telegram import Message  # pylint: disable=import-outside-toplevel  # noqa: PLC0415

        data["checklist_message"] = de_json_optional(data.get("checklist_message"), Message, bot)
        data["tasks"] = ChecklistTask.de_list(data.get("tasks", []), bot)

        return super().de_json(data=data, bot=bot)



#!/usr/bin/env python
#
# A library that provides a Python interface to the Telegram Bot API
# Copyright (C) 2015-2026
# Leandro Toledo de Souza <devs@python-telegram-bot.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser Public License for more details.
#
# You should have received a copy of the GNU Lesser Public License
# along with this program. If not, see [http://www.gnu.org/licenses/].

import pytest

from telegram import ChatShared, PhotoSize, SharedUser, UsersShared
from tests.auxil.slots import mro_slots


@pytest.fixture(scope="class")
def users_shared():
    return UsersShared(UsersSharedTestBase.request_id, users=UsersSharedTestBase.users)


class UsersSharedTestBase:
    request_id = 789
    user_ids = (101112, 101113)
    users = (SharedUser(101112, "user1"), SharedUser(101113, "user2"))


class TestUsersSharedWithoutRequest(UsersSharedTestBase):
    def test_slot_behaviour(self, users_shared):
        for attr in users_shared.__slots__:
            assert getattr(users_shared, attr, "err") != "err", f"got extra slot '{attr}'"
        assert len(mro_slots(users_shared)) == len(set(mro_slots(users_shared))), "duplicate slot"

    def test_to_dict(self, users_shared):
        users_shared_dict = users_shared.to_dict()

        assert isinstance(users_shared_dict, dict)
        assert users_shared_dict["request_id"] == self.request_id
        assert users_shared_dict["users"] == [user.to_dict() for user in self.users]

    def test_de_json(self, offline_bot):
        json_dict = {
            "request_id": self.request_id,
            "users": [user.to_dict() for user in self.users],
            "user_ids": self.user_ids,
        }
        users_shared = UsersShared.de_json(json_dict, offline_bot)
        assert users_shared.api_kwargs == {"user_ids": self.user_ids}

        assert users_shared.request_id == self.request_id
        assert users_shared.users == self.users

    def test_equality(self):
        a = UsersShared(self.request_id, users=self.users)
        b = UsersShared(self.request_id, users=self.users)
        c = UsersShared(1, users=self.users)
        d = UsersShared(self.request_id, users=(SharedUser(1, "user1"), SharedUser(1, "user2")))
        e = PhotoSize("file_id", "1", 1, 1)

        assert a == b
        assert hash(a) == hash(b)
        assert a is not b

        assert a != c
        assert hash(a) != hash(c)

        assert a != d
        assert hash(a) != hash(d)

        assert a != e
        assert hash(a) != hash(e)


@pytest.fixture(scope="class")
def chat_shared():
    return ChatShared(
        ChatSharedTestBase.request_id,
        ChatSharedTestBase.chat_id,
    )


class ChatSharedTestBase:
    request_id = 131415
    chat_id = 161718


class TestChatSharedWithoutRequest(ChatSharedTestBase):
    def test_slot_behaviour(self, chat_shared):
        for attr in chat_shared.__slots__:
            assert getattr(chat_shared, attr, "err") != "err", f"got extra slot '{attr}'"
        assert len(mro_slots(chat_shared)) == len(set(mro_slots(chat_shared))), "duplicate slot"

    def test_to_dict(self, chat_shared):
        chat_shared_dict = chat_shared.to_dict()

        assert isinstance(chat_shared_dict, dict)
        assert chat_shared_dict["request_id"] == self.request_id
        assert chat_shared_dict["chat_id"] == self.chat_id

    def test_de_json(self, offline_bot):
        json_dict = {
            "request_id": self.request_id,
            "chat_id": self.chat_id,
        }
        chat_shared = ChatShared.de_json(json_dict, offline_bot)
        assert chat_shared.api_kwargs == {}

        assert chat_shared.request_id == self.request_id
        assert chat_shared.chat_id == self.chat_id

    def test_link(self):
        chat_shared = ChatShared(1, 123, username="username")
        assert chat_shared.link == f"https://t.me/{chat_shared.username}"
        chat_shared = ChatShared(1, 123)
        assert chat_shared.link is None

    def test_equality(self, users_shared):
        a = ChatShared(self.request_id, self.chat_id)
        b = ChatShared(self.request_id, self.chat_id)
        c = ChatShared(1, self.chat_id)
        d = ChatShared(self.request_id, 1)
        e = users_shared

        assert a == b
        assert hash(a) == hash(b)
        assert a is not b

        assert a != c
        assert hash(a) != hash(c)

        assert a != d
        assert hash(a) != hash(d)

        assert a != e
        assert hash(a) != hash(e)


@pytest.fixture(scope="class")
def shared_user():
    return SharedUser(
        SharedUserTestBase.user_id,
        SharedUserTestBase.first_name,
        last_name=SharedUserTestBase.last_name,
        username=SharedUserTestBase.username,
        photo=SharedUserTestBase.photo,
    )


class SharedUserTestBase:
    user_id = 101112
    first_name = "first"
    last_name = "last"
    username = "user"
    photo = (
        PhotoSize(file_id="file_id", width=1, height=1, file_unique_id="1"),
        PhotoSize(file_id="file_id", width=2, height=2, file_unique_id="2"),
    )


class TestSharedUserWithoutRequest(SharedUserTestBase):
    def test_slot_behaviour(self, shared_user):
        for attr in shared_user.__slots__:
            assert getattr(shared_user, attr, "err") != "err", f"got extra slot '{attr}'"
        assert len(mro_slots(shared_user)) == len(set(mro_slots(shared_user))), "duplicate slot"

    def test_to_dict(self, shared_user):
        shared_user_dict = shared_user.to_dict()

        assert isinstance(shared_user_dict, dict)
        assert shared_user_dict["user_id"] == self.user_id
        assert shared_user_dict["first_name"] == self.first_name
        assert shared_user_dict["last_name"] == self.last_name
        assert shared_user_dict["username"] == self.username
        assert shared_user_dict["photo"] == [photo.to_dict() for photo in self.photo]

    def test_de_json_required(self, offline_bot):
        json_dict = {
            "user_id": self.user_id,
            "first_name": self.first_name,
        }
        shared_user = SharedUser.de_json(json_dict, offline_bot)
        assert shared_user.api_kwargs == {}

        assert shared_user.user_id == self.user_id
        assert shared_user.first_name == self.first_name
        assert shared_user.last_name is None
        assert shared_user.username is None
        assert shared_user.photo == ()

    def test_de_json_all(self, offline_bot):
        json_dict = {
            "user_id": self.user_id,
            "first_name": self.first_name,
            "last_name": self.last_name,
            "username": self.username,
            "photo": [photo.to_dict() for photo in self.photo],
        }
        shared_user = SharedUser.de_json(json_dict, offline_bot)
        assert shared_user.api_kwargs == {}

        assert shared_user.user_id == self.user_id
        assert shared_user.first_name == self.first_name
        assert shared_user.last_name == self.last_name
        assert shared_user.username == self.username
        assert shared_user.photo == self.photo

    def test_name(self):
        shared_user = SharedUser(123, "first_name", "last_name", "username")
        assert shared_user.name == f"@{shared_user.username}"
        shared_user = SharedUser(123, "first_name", "last_name")
        assert shared_user.name == f"{shared_user.first_name} {shared_user.last_name}"
        shared_user = SharedUser(123, "first_name")
        assert shared_user.name == f"{shared_user.first_name}"
        shared_user = SharedUser(123, "first_name", username="username")
        assert shared_user.name == f"@{shared_user.username}"
        shared_user = SharedUser(123)
        assert shared_user.name is None

    def test_full_name(self):
        shared_user = SharedUser(123, "first_name", "last_name")
        assert shared_user.full_name == f"{shared_user.first_name} {shared_user.last_name}"
        shared_user = SharedUser(123, "first_name")
        assert shared_user.full_name == f"{shared_user.first_name}"
        shared_user = SharedUser(123)
        assert shared_user.full_name is None

    def test_link(self):
        shared_user = SharedUser(123, username="username")
        assert shared_user.link == f"https://t.me/{shared_user.username}"
        shared_user = SharedUser(123, "first_name", "last_name")
        assert shared_user.link is None

    def test_equality(self, chat_shared):
        a = SharedUser(
            self.user_id,
            self.first_name,
            last_name=self.last_name,
            username=self.username,
            photo=self.photo,
        )
        b = SharedUser(self.user_id, "other_firs_name")
        c = SharedUser(self.user_id + 1, self.first_name)
        d = chat_shared

        assert a == b
        assert hash(a) == hash(b)
        assert a is not b

        assert a != c
        assert hash(a) != hash(c)

        assert a != d
        assert hash(a) != hash(d)



#!/usr/bin/env python
#
# A library that provides a Python interface to the Telegram Bot API
# Copyright (C) 2015-2026
# Leandro Toledo de Souza <devs@python-telegram-bot.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser Public License for more details.
#
# You should have received a copy of the GNU Lesser Public License
# along with this program.  If not, see [http://www.gnu.org/licenses/].

import datetime as dtm

import pytest

from telegram import (
    BotCommand,
    Chat,
    Sticker,
    UniqueGift,
    UniqueGiftBackdrop,
    UniqueGiftBackdropColors,
    UniqueGiftColors,
    UniqueGiftInfo,
    UniqueGiftModel,
    UniqueGiftSymbol,
)
from telegram._utils.datetime import UTC, to_timestamp
from telegram.constants import UniqueGiftInfoOrigin, UniqueGiftModelRarity
from tests.auxil.slots import mro_slots


@pytest.fixture
def unique_gift_colors():
    return UniqueGiftColors(
        model_custom_emoji_id=UniqueGiftColorsTestBase.model_custom_emoji_id,
        symbol_custom_emoji_id=UniqueGiftColorsTestBase.symbol_custom_emoji_id,
        light_theme_main_color=UniqueGiftColorsTestBase.light_theme_main_color,
        light_theme_other_colors=UniqueGiftColorsTestBase.light_theme_other_colors,
        dark_theme_main_color=UniqueGiftColorsTestBase.dark_theme_main_color,
        dark_theme_other_colors=UniqueGiftColorsTestBase.dark_theme_other_colors,
    )


class UniqueGiftColorsTestBase:
    model_custom_emoji_id = "model_emoji_id"
    symbol_custom_emoji_id = "symbol_emoji_id"
    light_theme_main_color = 0xFFFFFF
    light_theme_other_colors = [0xAAAAAA, 0xBBBBBB]
    dark_theme_main_color = 0x000000
    dark_theme_other_colors = [0x111111, 0x222222]


class TestUniqueGiftColorsWithoutRequest(UniqueGiftColorsTestBase):
    def test_slot_behaviour(self, unique_gift_colors):
        for attr in unique_gift_colors.__slots__:
            assert getattr(unique_gift_colors, attr, "err") != "err", f"got extra slot '{attr}'"
        assert len(mro_slots(unique_gift_colors)) == len(set(mro_slots(unique_gift_colors))), (
            "duplicate slot"
        )

    def test_de_json(self, offline_bot):
        json_dict = {
            "model_custom_emoji_id": self.model_custom_emoji_id,
            "symbol_custom_emoji_id": self.symbol_custom_emoji_id,
            "light_theme_main_color": self.light_theme_main_color,
            "light_theme_other_colors": self.light_theme_other_colors,
            "dark_theme_main_color": self.dark_theme_main_color,
            "dark_theme_other_colors": self.dark_theme_other_colors,
        }
        unique_gift_colors = UniqueGiftColors.de_json(json_dict, offline_bot)
        assert unique_gift_colors.api_kwargs == {}
        assert unique_gift_colors.model_custom_emoji_id == self.model_custom_emoji_id
        assert unique_gift_colors.symbol_custom_emoji_id == self.symbol_custom_emoji_id
        assert unique_gift_colors.light_theme_main_color == self.light_theme_main_color
        assert unique_gift_colors.light_theme_other_colors == tuple(self.light_theme_other_colors)
        assert unique_gift_colors.dark_theme_main_color == self.dark_theme_main_color
        assert unique_gift_colors.dark_theme_other_colors == tuple(self.dark_theme_other_colors)

    def test_to_dict(self, unique_gift_colors):
        json_dict = unique_gift_colors.to_dict()
        assert json_dict["model_custom_emoji_id"] == self.model_custom_emoji_id
        assert json_dict["symbol_custom_emoji_id"] == self.symbol_custom_emoji_id
        assert json_dict["light_theme_main_color"] == self.light_theme_main_color
        assert json_dict["light_theme_other_colors"] == self.light_theme_other_colors
        assert json_dict["dark_theme_main_color"] == self.dark_theme_main_color
        assert json_dict["dark_theme_other_colors"] == self.dark_theme_other_colors

    def test_equality(self, unique_gift_colors):
        a = unique_gift_colors
        b = UniqueGiftColors(
            self.model_custom_emoji_id,
            self.symbol_custom_emoji_id,
            self.light_theme_main_color,
            self.light_theme_other_colors,
            self.dark_theme_main_color,
            self.dark_theme_other_colors,
        )
        c = UniqueGiftColors(
            "other_model_emoji_id",
            self.symbol_custom_emoji_id,
            self.light_theme_main_color,
            self.light_theme_other_colors,
            self.dark_theme_main_color,
            self.dark_theme_other_colors,
        )
        d = BotCommand("start", "description")

        assert a == b
        assert hash(a) == hash(b)

        assert a != c
        assert hash(a) != hash(c)

        assert a != d
        assert hash(a) != hash(d)


@pytest.fixture
def unique_gift():
    return UniqueGift(
        gift_id=UniqueGiftTestBase.gift_id,
        base_name=UniqueGiftTestBase.base_name,
        name=UniqueGiftTestBase.name,
        number=UniqueGiftTestBase.number,
        model=UniqueGiftTestBase.model,
        symbol=UniqueGiftTestBase.symbol,
        backdrop=UniqueGiftTestBase.backdrop,
        publisher_chat=UniqueGiftTestBase.publisher_chat,
        is_premium=UniqueGiftTestBase.is_premium,
        is_from_blockchain=UniqueGiftTestBase.is_from_blockchain,
        colors=UniqueGiftTestBase.colors,
        is_burned=UniqueGiftTestBase.is_burned,
    )


class UniqueGiftTestBase:
    gift_id = "gift_id"
    base_name = "human_readable"
    name = "unique_name"
    number = 10
    model = UniqueGiftModel(
        name="model_name",
        sticker=Sticker("file_id1", "file_unique_id1", 512, 512, False, False, "regular"),
        rarity_per_mille=10,
    )
    symbol = UniqueGiftSymbol(
        name="symbol_name",
        sticker=Sticker("file_id2", "file_unique_id2", 512, 512, True, True, "mask"),
        rarity_per_mille=20,
    )
    backdrop = UniqueGiftBackdrop(
        name="backdrop_name",
        colors=UniqueGiftBackdropColors(0x00FF00, 0xEE00FF, 0xAA22BB, 0x20FE8F),
        rarity_per_mille=30,
    )
    publisher_chat = Chat(1, Chat.PRIVATE)
    is_premium = False
    is_from_blockchain = True
    colors = UniqueGiftColors(
        model_custom_emoji_id="M",
        symbol_custom_emoji_id="S",
        light_theme_main_color=0xFFFFFF,
        light_theme_other_colors=[0xAAAAAA],
        dark_theme_main_color=0x000000,
        dark_theme_other_colors=[0x111111],
    )
    is_burned = True


class TestUniqueGiftWithoutRequest(UniqueGiftTestBase):
    def test_slot_behaviour(self, unique_gift):
        for attr in unique_gift.__slots__:
            assert getattr(unique_gift, attr, "err") != "err", f"got extra slot '{attr}'"
        assert len(mro_slots(unique_gift)) == len(set(mro_slots(unique_gift))), "duplicate slot"

    def test_de_json(self, offline_bot):
        json_dict = {
            "gift_id": self.gift_id,
            "base_name": self.base_name,
            "name": self.name,
            "number": self.number,
            "model": self.model.to_dict(),
            "symbol": self.symbol.to_dict(),
            "backdrop": self.backdrop.to_dict(),
            "publisher_chat": self.publisher_chat.to_dict(),
            "is_premium": self.is_premium,
            "is_from_blockchain": self.is_from_blockchain,
            "colors": self.colors.to_dict(),
            "is_burned": self.is_burned,
        }
        unique_gift = UniqueGift.de_json(json_dict, offline_bot)
        assert unique_gift.api_kwargs == {}

        assert unique_gift.base_name == self.base_name
        assert unique_gift.name == self.name
        assert unique_gift.number == self.number
        assert unique_gift.model == self.model
        assert unique_gift.symbol == self.symbol
        assert unique_gift.backdrop == self.backdrop
        assert unique_gift.publisher_chat == self.publisher_chat
        assert unique_gift.is_premium == self.is_premium
        assert unique_gift.is_from_blockchain == self.is_from_blockchain
        assert unique_gift.colors == self.colors
        assert unique_gift.is_burned == self.is_burned

    def test_to_dict(self, unique_gift):
        gift_dict = unique_gift.to_dict()

        assert isinstance(gift_dict, dict)
        assert gift_dict["gift_id"] == self.gift_id
        assert gift_dict["base_name"] == self.base_name
        assert gift_dict["name"] == self.name
        assert gift_dict["number"] == self.number
        assert gift_dict["model"] == self.model.to_dict()
        assert gift_dict["symbol"] == self.symbol.to_dict()
        assert gift_dict["backdrop"] == self.backdrop.to_dict()
        assert gift_dict["publisher_chat"] == self.publisher_chat.to_dict()
        assert gift_dict["is_premium"] == self.is_premium
        assert gift_dict["is_from_blockchain"] == self.is_from_blockchain
        assert gift_dict["colors"] == self.colors.to_dict()
        assert gift_dict["is_burned"] == self.is_burned

    def test_equality(self, unique_gift):
        a = unique_gift
        b = UniqueGift(
            gift_id=self.gift_id,
            base_name=self.base_name,
            name=self.name,
            number=self.number,
            model=self.model,
            symbol=self.symbol,
            backdrop=self.backdrop,
            publisher_chat=self.publisher_chat,
            is_burned=self.is_burned,
        )
        c = UniqueGift(
            gift_id=self.gift_id,
            base_name="other_base_name",
            name=self.name,
            number=self.number,
            model=self.model,
            symbol=self.symbol,
            backdrop=self.backdrop,
            publisher_chat=self.publisher_chat,
        )
        d = BotCommand("start", "description")

        assert a == b
        assert hash(a) == hash(b)

        assert a != c
        assert hash(a) != hash(c)

        assert a != d
        assert hash(a) != hash(d)


@pytest.fixture
def unique_gift_model():
    return UniqueGiftModel(
        name=UniqueGiftModelTestBase.name,
        sticker=UniqueGiftModelTestBase.sticker,
        rarity_per_mille=UniqueGiftModelTestBase.rarity_per_mille,
        rarity=UniqueGiftModelTestBase.rarity,
    )


class UniqueGiftModelTestBase:
    name = "model_name"
    sticker = Sticker("file_id", "file_unique_id", 512, 512, False, False, "regular")
    rarity_per_mille = 10
    rarity = UniqueGiftModelRarity.UNCOMMON


class TestUniqueGiftModelWithoutRequest(UniqueGiftModelTestBase):
    def test_slot_behaviour(self, unique_gift_model):
        for attr in unique_gift_model.__slots__:
            assert getattr(unique_gift_model, attr, "err") != "err", f"got extra slot '{attr}'"
        assert len(mro_slots(unique_gift_model)) == len(set(mro_slots(unique_gift_model))), (
            "duplicate slot"
        )

    def test_de_json(self, offline_bot):
        json_dict = {
            "name": self.name,
            "sticker": self.sticker.to_dict(),
            "rarity_per_mille": self.rarity_per_mille,
            "rarity": self.rarity,
        }
        unique_gift_model = UniqueGiftModel.de_json(json_dict, offline_bot)
        assert unique_gift_model.api_kwargs == {}
        assert unique_gift_model.name == self.name
        assert unique_gift_model.sticker == self.sticker
        assert unique_gift_model.rarity_per_mille == self.rarity_per_mille
        assert unique_gift_model.rarity == self.rarity

    def test_to_dict(self, unique_gift_model):
        json_dict = unique_gift_model.to_dict()
        assert json_dict["name"] == self.name
        assert json_dict["sticker"] == self.sticker.to_dict()
        assert json_dict["rarity_per_mille"] == self.rarity_per_mille
        assert json_dict["rarity"] == self.rarity.value

    def test_enum_type_conversion(self, unique_gift_model):
        assert type(unique_gift_model.rarity) is UniqueGiftModelRarity
        assert unique_gift_model.rarity == UniqueGiftModelRarity.UNCOMMON

    def test_equality(self, unique_gift_model):
        a = unique_gift_model
        b = UniqueGiftModel(self.name, self.sticker, self.rarity_per_mille)
        c = UniqueGiftModel("other_name", self.sticker, self.rarity_per_mille)
        d = UniqueGiftSymbol(self.name, self.sticker, self.rarity_per_mille)

        assert a == b
        assert hash(a) == hash(b)

        assert a != c
        assert hash(a) != hash(c)

        assert a != d
        assert hash(a) != hash(d)


@pytest.fixture
def unique_gift_symbol():
    return UniqueGiftSymbol(
        name=UniqueGiftSymbolTestBase.name,
        sticker=UniqueGiftSymbolTestBase.sticker,
        rarity_per_mille=UniqueGiftSymbolTestBase.rarity_per_mille,
    )


class UniqueGiftSymbolTestBase:
    name = "symbol_name"
    sticker = Sticker("file_id", "file_unique_id", 512, 512, False, False, "regular")
    rarity_per_mille = 20


class TestUniqueGiftSymbolWithoutRequest(UniqueGiftSymbolTestBase):
    def test_slot_behaviour(self, unique_gift_symbol):
        for attr in unique_gift_symbol.__slots__:
            assert getattr(unique_gift_symbol, attr, "err") != "err", f"got extra slot '{attr}'"
        assert len(mro_slots(unique_gift_symbol)) == len(set(mro_slots(unique_gift_symbol))), (
            "duplicate slot"
        )

    def test_de_json(self, offline_bot):
        json_dict = {
            "name": self.name,
            "sticker": self.sticker.to_dict(),
            "rarity_per_mille": self.rarity_per_mille,
        }
        unique_gift_symbol = UniqueGiftSymbol.de_json(json_dict, offline_bot)
        assert unique_gift_symbol.api_kwargs == {}
        assert unique_gift_symbol.name == self.name
        assert unique_gift_symbol.sticker == self.sticker
        assert unique_gift_symbol.rarity_per_mille == self.rarity_per_mille

    def test_to_dict(self, unique_gift_symbol):
        json_dict = unique_gift_symbol.to_dict()
        assert json_dict["name"] == self.name
        assert json_dict["sticker"] == self.sticker.to_dict()
        assert json_dict["rarity_per_mille"] == self.rarity_per_mille

    def test_equality(self, unique_gift_symbol):
        a = unique_gift_symbol
        b = UniqueGiftSymbol(self.name, self.sticker, self.rarity_per_mille)
        c = UniqueGiftSymbol("other_name", self.sticker, self.rarity_per_mille)
        d = UniqueGiftModel(self.name, self.sticker, self.rarity_per_mille)

        assert a == b
        assert hash(a) == hash(b)

        assert a != c
        assert hash(a) != hash(c)

        assert a != d
        assert hash(a) != hash(d)


@pytest.fixture
def unique_gift_backdrop():
    return UniqueGiftBackdrop(
        name=UniqueGiftBackdropTestBase.name,
        colors=UniqueGiftBackdropTestBase.colors,
        rarity_per_mille=UniqueGiftBackdropTestBase.rarity_per_mille,
    )


class UniqueGiftBackdropTestBase:
    name = "backdrop_name"
    colors = UniqueGiftBackdropColors(0x00FF00, 0xEE00FF, 0xAA22BB, 0x20FE8F)
    rarity_per_mille = 30


class TestUniqueGiftBackdropWithoutRequest(UniqueGiftBackdropTestBase):
    def test_slot_behaviour(self, unique_gift_backdrop):
        for attr in unique_gift_backdrop.__slots__:
            assert getattr(unique_gift_backdrop, attr, "err") != "err", f"got extra slot '{attr}'"
        assert len(mro_slots(unique_gift_backdrop)) == len(set(mro_slots(unique_gift_backdrop))), (
            "duplicate slot"
        )

    def test_de_json(self, offline_bot):
        json_dict = {
            "name": self.name,
            "colors": self.colors.to_dict(),
            "rarity_per_mille": self.rarity_per_mille,
        }
        unique_gift_backdrop = UniqueGiftBackdrop.de_json(json_dict, offline_bot)
        assert unique_gift_backdrop.api_kwargs == {}
        assert unique_gift_backdrop.name == self.name
        assert unique_gift_backdrop.colors == self.colors
        assert unique_gift_backdrop.rarity_per_mille == self.rarity_per_mille

    def test_to_dict(self, unique_gift_backdrop):
        json_dict = unique_gift_backdrop.to_dict()
        assert json_dict["name"] == self.name
        assert json_dict["colors"] == self.colors.to_dict()
        assert json_dict["rarity_per_mille"] == self.rarity_per_mille

    def test_equality(self, unique_gift_backdrop):
        a = unique_gift_backdrop
        b = UniqueGiftBackdrop(self.name, self.colors, self.rarity_per_mille)
        c = UniqueGiftBackdrop("other_name", self.colors, self.rarity_per_mille)
        d = BotCommand("start", "description")

        assert a == b
        assert hash(a) == hash(b)

        assert a != c
        assert hash(a) != hash(c)

        assert a != d
        assert hash(a) != hash(d)


@pytest.fixture
def unique_gift_backdrop_colors():
    return UniqueGiftBackdropColors(
        center_color=UniqueGiftBackdropColorsTestBase.center_color,
        edge_color=UniqueGiftBackdropColorsTestBase.edge_color,
        symbol_color=UniqueGiftBackdropColorsTestBase.symbol_color,
        text_color=UniqueGiftBackdropColorsTestBase.text_color,
    )


class UniqueGiftBackdropColorsTestBase:
    center_color = 0x00FF00
    edge_color = 0xEE00FF
    symbol_color = 0xAA22BB
    text_color = 0x20FE8F


class TestUniqueGiftBackdropColorsWithoutRequest(UniqueGiftBackdropColorsTestBase):
    def test_slot_behaviour(self, unique_gift_backdrop_colors):
        for attr in unique_gift_backdrop_colors.__slots__:
            assert getattr(unique_gift_backdrop_colors, attr, "err") != "err", (
                f"got extra slot '{attr}'"
            )
        assert len(mro_slots(unique_gift_backdrop_colors)) == len(
            set(mro_slots(unique_gift_backdrop_colors))
        ), "duplicate slot"

    def test_de_json(self, offline_bot):
        json_dict = {
            "center_color": self.center_color,
            "edge_color": self.edge_color,
            "symbol_color": self.symbol_color,
            "text_color": self.text_color,
        }
        unique_gift_backdrop_colors = UniqueGiftBackdropColors.de_json(json_dict, offline_bot)
        assert unique_gift_backdrop_colors.api_kwargs == {}
        assert unique_gift_backdrop_colors.center_color == self.center_color
        assert unique_gift_backdrop_colors.edge_color == self.edge_color
        assert unique_gift_backdrop_colors.symbol_color == self.symbol_color
        assert unique_gift_backdrop_colors.text_color == self.text_color

    def test_to_dict(self, unique_gift_backdrop_colors):
        json_dict = unique_gift_backdrop_colors.to_dict()
        assert json_dict["center_color"] == self.center_color
        assert json_dict["edge_color"] == self.edge_color
        assert json_dict["symbol_color"] == self.symbol_color
        assert json_dict["text_color"] == self.text_color

    def test_equality(self, unique_gift_backdrop_colors):
        a = unique_gift_backdrop_colors
        b = UniqueGiftBackdropColors(
            center_color=self.center_color,
            edge_color=self.edge_color,
            symbol_color=self.symbol_color,
            text_color=self.text_color,
        )
        c = UniqueGiftBackdropColors(
            center_color=0x000000,
            edge_color=self.edge_color,
            symbol_color=self.symbol_color,
            text_color=self.text_color,
        )
        d = BotCommand("start", "description")

        assert a == b
        assert hash(a) == hash(b)

        assert a != c
        assert hash(a) != hash(c)

        assert a != d
        assert hash(a) != hash(d)


@pytest.fixture
def unique_gift_info():
    return UniqueGiftInfo(
        gift=UniqueGiftInfoTestBase.gift,
        origin=UniqueGiftInfoTestBase.origin,
        owned_gift_id=UniqueGiftInfoTestBase.owned_gift_id,
        transfer_star_count=UniqueGiftInfoTestBase.transfer_star_count,
        last_resale_currency=UniqueGiftInfoTestBase.last_resale_currency,
        last_resale_amount=UniqueGiftInfoTestBase.last_resale_amount,
        next_transfer_date=UniqueGiftInfoTestBase.next_transfer_date,
    )


class UniqueGiftInfoTestBase:
    gift = UniqueGift(
        gift_id="gift_id",
        base_name="human_readable_name",
        name="unique_name",
        number=10,
        model=UniqueGiftModel(
            name="model_name",
            sticker=Sticker("file_id1", "file_unique_id1", 512, 512, False, False, "regular"),
            rarity_per_mille=10,
        ),
        symbol=UniqueGiftSymbol(
            name="symbol_name",
            sticker=Sticker("file_id2", "file_unique_id2", 512, 512, True, True, "mask"),
            rarity_per_mille=20,
        ),
        backdrop=UniqueGiftBackdrop(
            name="backdrop_name",
            colors=UniqueGiftBackdropColors(0x00FF00, 0xEE00FF, 0xAA22BB, 0x20FE8F),
            rarity_per_mille=2,
        ),
    )
    origin = UniqueGiftInfo.UPGRADE
    owned_gift_id = "some_id"
    transfer_star_count = 10
    last_resale_currency = "XTR"
    last_resale_amount = 1234
    next_transfer_date = dtm.datetime.now(tz=UTC).replace(microsecond=0)


class TestUniqueGiftInfoWithoutRequest(UniqueGiftInfoTestBase):
    def test_slot_behaviour(self, unique_gift_info):
        for attr in unique_gift_info.__slots__:
            assert getattr(unique_gift_info, attr, "err") != "err", f"got extra slot '{attr}'"
        assert len(mro_slots(unique_gift_info)) == len(set(mro_slots(unique_gift_info))), (
            "duplicate slot"
        )

    def test_de_json(self, offline_bot):
        json_dict = {
            "gift": self.gift.to_dict(),
            "origin": self.origin,
            "owned_gift_id": self.owned_gift_id,
            "transfer_star_count": self.transfer_star_count,
            "last_resale_currency": self.last_resale_currency,
            "last_resale_amount": self.last_resale_amount,
            "next_transfer_date": to_timestamp(self.next_transfer_date),
        }
        unique_gift_info = UniqueGiftInfo.de_json(json_dict, offline_bot)
        assert unique_gift_info.api_kwargs == {}
        assert unique_gift_info.gift == self.gift
        assert unique_gift_info.origin == self.origin
        assert unique_gift_info.owned_gift_id == self.owned_gift_id
        assert unique_gift_info.transfer_star_count == self.transfer_star_count
        assert unique_gift_info.last_resale_currency == self.last_resale_currency
        assert unique_gift_info.last_resale_amount == self.last_resale_amount
        assert unique_gift_info.next_transfer_date == self.next_transfer_date

    def test_de_json_localization(self, tz_bot, offline_bot, raw_bot):
        json_dict = {
            "gift": self.gift.to_dict(),
            "origin": self.origin,
            "owned_gift_id": self.owned_gift_id,
            "transfer_star_count": self.transfer_star_count,
            "last_resale_currency": self.last_resale_currency,
            "last_resale_amount": self.last_resale_amount,
            "next_transfer_date": to_timestamp(self.next_transfer_date),
        }

        unique_gift_info_raw = UniqueGiftInfo.de_json(json_dict, raw_bot)
        unique_gift_info_offline = UniqueGiftInfo.de_json(json_dict, offline_bot)
        unique_gift_info_tz = UniqueGiftInfo.de_json(json_dict, tz_bot)

        # comparing utcoffsets because comparing timezones is unpredicatable
        unique_gift_info_tz_offset = unique_gift_info_tz.next_transfer_date.utcoffset()
        tz_bot_offset = tz_bot.defaults.tzinfo.utcoffset(
            unique_gift_info_tz.next_transfer_date.replace(tzinfo=None)
        )

        assert unique_gift_info_raw.next_transfer_date.tzinfo == UTC
        assert unique_gift_info_offline.next_transfer_date.tzinfo == UTC
        assert unique_gift_info_tz_offset == tz_bot_offset

    def test_to_dict(self, unique_gift_info):
        json_dict = unique_gift_info.to_dict()
        assert json_dict["gift"] == self.gift.to_dict()
        assert json_dict["origin"] == self.origin
        assert json_dict["owned_gift_id"] == self.owned_gift_id
        assert json_dict["transfer_star_count"] == self.transfer_star_count
        assert json_dict["last_resale_currency"] == self.last_resale_currency
        assert json_dict["last_resale_amount"] == self.last_resale_amount
        assert json_dict["next_transfer_date"] == to_timestamp(self.next_transfer_date)

    def test_enum_type_conversion(self, unique_gift_info):
        assert type(unique_gift_info.origin) is UniqueGiftInfoOrigin
        assert unique_gift_info.origin == UniqueGiftInfoOrigin.UPGRADE

    def test_equality(self, unique_gift_info):
        a = unique_gift_info
        b = UniqueGiftInfo(self.gift, self.origin, self.owned_gift_id, self.transfer_star_count)
        c = UniqueGiftInfo(
            self.gift, UniqueGiftInfo.TRANSFER, self.owned_gift_id, self.transfer_star_count
        )
        d = BotCommand("start", "description")

        assert a == b
        assert hash(a) == hash(b)

        assert a != c
        assert hash(a) != hash(c)

        assert a != d
        assert hash(a) != hash(d)



#!/usr/bin/env python
#
# A library that provides a Python interface to the Telegram Bot API
# Copyright (C) 2015-2026
# Leandro Toledo de Souza <devs@python-telegram-bot.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser Public License for more details.
#
# You should have received a copy of the GNU Lesser Public License
# along with this program.  If not, see [http://www.gnu.org/licenses/].
"""This module contains helper functions for the official API tests used in the other modules."""

import functools
import re
from collections.abc import Sequence
from typing import TYPE_CHECKING, Any, TypeVar, _eval_type, get_type_hints

from bs4 import PageElement, Tag

import telegram
import telegram._utils.defaultvalue
import telegram._utils.types

if TYPE_CHECKING:
    from tests.test_official.scraper import TelegramParameter


tg_objects = vars(telegram)
tg_objects.update(vars(telegram._utils.types))
tg_objects.update(vars(telegram._utils.defaultvalue))


def _get_params_base(object_name: str, search_dict: dict[str, set[Any]]) -> set[Any]:
    """Helper function for the *_params functions below.
    Given an object name and a search dict, goes through the keys of the search dict and checks if
    the object name matches any of the regexes (keys). The union of all the sets (values) of the
    matching regexes is returned. `object_name` may be a CamelCase or snake_case name.
    """
    out = set()
    for regex, params in search_dict.items():
        if re.fullmatch(regex, object_name):
            out.update(params)
        # also check the snake_case version
        snake_case_name = re.sub(r"(?<!^)(?=[A-Z])", "_", object_name).lower()
        if re.fullmatch(regex, snake_case_name):
            out.update(params)
    return out


def _extract_words(text: str) -> set[str]:
    """Extracts all words from a string, removing all punctuation and words like 'and' & 'or'."""
    return set(re.sub(r"[^\w\s]", "", text).split()) - {"and", "or"}


def _unionizer(annotation: Sequence[Any] | set[Any]) -> Any:
    """Returns a union of all the types in the annotation. Also imports objects from lib."""
    union = None
    for t in annotation:
        if isinstance(t, str):  # we have to import objects from lib
            t = getattr(telegram, t)  # noqa: PLW2901
        union = t if union is None else union | t
    return union


def find_next_sibling_until(tag: Tag, name: str, until: Tag) -> PageElement | None:
    for sibling in tag.next_siblings:
        if sibling is until:
            return None
        if sibling.name == name:
            return sibling
    return None


def is_pascal_case(s):
    "PascalCase. Starts with a capital letter and has no spaces. Useful for identifying classes."
    return bool(re.match(r"^[A-Z][a-zA-Z\d]*$", s))


def is_parameter_required_by_tg(field: str) -> bool:
    if field in {"Required", "Yes"}:
        return True
    return field.split(".", 1)[0] != "Optional"  # splits the sentence and extracts first word


def wrap_with_none(tg_parameter: "TelegramParameter", mapped_type: Any, obj: object) -> type:
    """Adds `None` to type annotation if the parameter isn't required. Respects ignored params."""
    # have to import here to avoid circular imports
    from tests.test_official.exceptions import ignored_param_requirements  # noqa: PLC0415

    if tg_parameter.param_name in ignored_param_requirements(obj.__name__):
        return mapped_type | type(None)
    return mapped_type | type(None) if not tg_parameter.param_required else mapped_type


@functools.cache
def cached_type_hints(obj: Any, is_class: bool) -> dict[str, Any]:
    """Returns type hints of a class, method, or function, with forward refs evaluated."""
    return get_type_hints(obj.__init__ if is_class else obj, localns=tg_objects)


@functools.cache
def resolve_forward_refs_in_type(obj: type) -> type:
    """Resolves forward references in a type hint."""
    return _eval_type(obj, localns=tg_objects, globalns=None)


T = TypeVar("T")


def extract_mappings(
    exceptions: dict[str, dict[str, T]], obj: object, param_name: str
) -> list[T] | None:
    mappings = (
        mapping for pattern, mapping in exceptions.items() if (re.match(pattern, obj.__name__))
    )
    out = [
        value
        for mapping in mappings
        for key, value in mapping.items()
        if re.match(key, param_name)
    ]

    return None or out



#!/usr/bin/env python
# pylint: disable=unused-argument
# This program is dedicated to the public domain under the CC0 license.

"""Bot that explains Telegram's "Deep Linking Parameters" functionality.

This program is dedicated to the public domain under the CC0 license.

This Bot uses the Application class to handle the bot.

First, a few handler functions are defined. Then, those functions are passed to
the Application and registered at their respective places.
Then, the bot is started and runs until we press Ctrl-C on the command line.

Usage:
Deep Linking example. Send /start to get the link.
Press Ctrl-C on the command line or send a signal to the process to stop the
bot.
"""

import logging

from telegram import (
    InlineKeyboardButton,
    InlineKeyboardMarkup,
    LinkPreviewOptions,
    Update,
    helpers,
)
from telegram.constants import ParseMode
from telegram.ext import Application, CallbackQueryHandler, CommandHandler, ContextTypes, filters

# Enable logging
logging.basicConfig(
    format="%(asctime)s - %(name)s - %(levelname)s - %(message)s", level=logging.INFO
)

# set higher logging level for httpx to avoid all GET and POST requests being logged
logging.getLogger("httpx").setLevel(logging.WARNING)

logger = logging.getLogger(__name__)

# Define constants that will allow us to reuse the deep-linking parameters.
CHECK_THIS_OUT = "check-this-out"
USING_ENTITIES = "using-entities-here"
USING_KEYBOARD = "using-keyboard-here"
SO_COOL = "so-cool"

# Callback data to pass in 3rd level deep-linking
KEYBOARD_CALLBACKDATA = "keyboard-callback-data"


async def start(update: Update, context: ContextTypes.DEFAULT_TYPE) -> None:
    """Send a deep-linked URL when the command /start is issued."""
    bot = context.bot
    url = helpers.create_deep_linked_url(bot.username, CHECK_THIS_OUT, group=True)
    text = "Feel free to tell your friends about it:\n\n" + url
    await update.message.reply_text(text)


async def deep_linked_level_1(update: Update, context: ContextTypes.DEFAULT_TYPE) -> None:
    """Reached through the CHECK_THIS_OUT payload"""
    bot = context.bot
    url = helpers.create_deep_linked_url(bot.username, SO_COOL)
    text = (
        "Awesome, you just accessed hidden functionality! Now let's get back to the private chat."
    )
    keyboard = InlineKeyboardMarkup.from_button(
        InlineKeyboardButton(text="Continue here!", url=url)
    )
    await update.message.reply_text(text, reply_markup=keyboard)


async def deep_linked_level_2(update: Update, context: ContextTypes.DEFAULT_TYPE) -> None:
    """Reached through the SO_COOL payload"""
    bot = context.bot
    url = helpers.create_deep_linked_url(bot.username, USING_ENTITIES)
    text = f'You can also mask the deep-linked URLs as links: <a href="{url}">▶️ CLICK HERE</a>.'
    await update.message.reply_text(
        text, parse_mode=ParseMode.HTML, link_preview_options=LinkPreviewOptions(is_disabled=True)
    )


async def deep_linked_level_3(update: Update, context: ContextTypes.DEFAULT_TYPE) -> None:
    """Reached through the USING_ENTITIES payload"""
    await update.message.reply_text(
        "It is also possible to make deep-linking using InlineKeyboardButtons.",
        reply_markup=InlineKeyboardMarkup(
            [[InlineKeyboardButton(text="Like this!", callback_data=KEYBOARD_CALLBACKDATA)]]
        ),
    )


async def deep_link_level_3_callback(update: Update, context: ContextTypes.DEFAULT_TYPE) -> None:
    """Answers CallbackQuery with deeplinking url."""
    bot = context.bot
    url = helpers.create_deep_linked_url(bot.username, USING_KEYBOARD)
    await update.callback_query.answer(url=url)


async def deep_linked_level_4(update: Update, context: ContextTypes.DEFAULT_TYPE) -> None:
    """Reached through the USING_KEYBOARD payload"""
    payload = context.args
    await update.message.reply_text(
        f"Congratulations! This is as deep as it gets 👏🏻\n\nThe payload was: {payload}"
    )


def main() -> None:
    """Start the bot."""
    # Create the Application and pass it your bot's token.
    application = Application.builder().token("TOKEN").build()

    # More info on what deep linking actually is (read this first if it's unclear to you):
    # https://core.telegram.org/bots/features#deep-linking

    # Register a deep-linking handler
    application.add_handler(
        CommandHandler("start", deep_linked_level_1, filters.Regex(CHECK_THIS_OUT))
    )

    # This one works with a textual link instead of an URL
    application.add_handler(CommandHandler("start", deep_linked_level_2, filters.Regex(SO_COOL)))

    # We can also pass on the deep-linking payload
    application.add_handler(
        CommandHandler("start", deep_linked_level_3, filters.Regex(USING_ENTITIES))
    )

    # Possible with inline keyboard buttons as well
    application.add_handler(
        CommandHandler("start", deep_linked_level_4, filters.Regex(USING_KEYBOARD))
    )

    # register callback handler for inline keyboard button
    application.add_handler(
        CallbackQueryHandler(deep_link_level_3_callback, pattern=KEYBOARD_CALLBACKDATA)
    )

    # Make sure the deep-linking handlers occur *before* the normal /start handler.
    application.add_handler(CommandHandler("start", start))

    # Run the bot until the user presses Ctrl-C
    application.run_polling(allowed_updates=Update.ALL_TYPES)


if __name__ == "__main__":
    main()



#!/usr/bin/env python
#
# A library that provides a Python interface to the Telegram Bot API
# Copyright (C) 2015-2026
# Leandro Toledo de Souza <devs@python-telegram-bot.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser Public License for more details.
#
# You should have received a copy of the GNU Lesser Public License
# along with this program.  If not, see [http://www.gnu.org/licenses/].

import pytest

from telegram import (
    InlineKeyboardButton,
    InlineKeyboardMarkup,
    InlineQueryResult,
    InlineQueryResultArticle,
    InlineQueryResultAudio,
    InputTextMessageContent,
)
from telegram.constants import InlineQueryResultType
from tests.auxil.slots import mro_slots


@pytest.fixture(scope="module")
def inline_query_result_article():
    return InlineQueryResultArticle(
        InlineQueryResultArticleTestBase.id_,
        InlineQueryResultArticleTestBase.title,
        input_message_content=InlineQueryResultArticleTestBase.input_message_content,
        reply_markup=InlineQueryResultArticleTestBase.reply_markup,
        url=InlineQueryResultArticleTestBase.url,
        description=InlineQueryResultArticleTestBase.description,
        thumbnail_url=InlineQueryResultArticleTestBase.thumbnail_url,
        thumbnail_height=InlineQueryResultArticleTestBase.thumbnail_height,
        thumbnail_width=InlineQueryResultArticleTestBase.thumbnail_width,
    )


class InlineQueryResultArticleTestBase:
    id_ = "id"
    type_ = "article"
    title = "title"
    input_message_content = InputTextMessageContent("input_message_content")
    reply_markup = InlineKeyboardMarkup([[InlineKeyboardButton("reply_markup")]])
    url = "url"
    description = "description"
    thumbnail_url = "thumb url"
    thumbnail_height = 10
    thumbnail_width = 15


class TestInlineQueryResultArticleWithoutRequest(InlineQueryResultArticleTestBase):
    def test_slot_behaviour(self, inline_query_result_article):
        inst = inline_query_result_article
        for attr in inst.__slots__:
            assert getattr(inst, attr, "err") != "err", f"got extra slot '{attr}'"
        assert len(mro_slots(inst)) == len(set(mro_slots(inst))), "duplicate slot"

    def test_expected_values(self, inline_query_result_article):
        assert inline_query_result_article.type == self.type_
        assert inline_query_result_article.id == self.id_
        assert inline_query_result_article.title == self.title
        assert (
            inline_query_result_article.input_message_content.to_dict()
            == self.input_message_content.to_dict()
        )
        assert inline_query_result_article.reply_markup.to_dict() == self.reply_markup.to_dict()
        assert inline_query_result_article.url == self.url
        assert inline_query_result_article.description == self.description
        assert inline_query_result_article.thumbnail_url == self.thumbnail_url
        assert inline_query_result_article.thumbnail_height == self.thumbnail_height
        assert inline_query_result_article.thumbnail_width == self.thumbnail_width

    def test_to_dict(self, inline_query_result_article):
        inline_query_result_article_dict = inline_query_result_article.to_dict()

        assert isinstance(inline_query_result_article_dict, dict)
        assert inline_query_result_article_dict["type"] == inline_query_result_article.type
        assert inline_query_result_article_dict["id"] == inline_query_result_article.id
        assert inline_query_result_article_dict["title"] == inline_query_result_article.title
        assert (
            inline_query_result_article_dict["input_message_content"]
            == inline_query_result_article.input_message_content.to_dict()
        )
        assert (
            inline_query_result_article_dict["reply_markup"]
            == inline_query_result_article.reply_markup.to_dict()
        )
        assert inline_query_result_article_dict["url"] == inline_query_result_article.url
        assert (
            inline_query_result_article_dict["description"]
            == inline_query_result_article.description
        )
        assert (
            inline_query_result_article_dict["thumbnail_url"]
            == inline_query_result_article.thumbnail_url
        )
        assert (
            inline_query_result_article_dict["thumbnail_height"]
            == inline_query_result_article.thumbnail_height
        )
        assert (
            inline_query_result_article_dict["thumbnail_width"]
            == inline_query_result_article.thumbnail_width
        )

    def test_type_enum_conversion(self):
        # Since we have a lot of different test files for all the result types, we test this
        # conversion only here. It is independent of the specific class
        assert (
            type(
                InlineQueryResult(
                    id="id",
                    type="article",
                ).type
            )
            is InlineQueryResultType
        )
        assert (
            InlineQueryResult(
                id="id",
                type="unknown",
            ).type
            == "unknown"
        )

    def test_equality(self):
        a = InlineQueryResultArticle(self.id_, self.title, self.input_message_content)
        b = InlineQueryResultArticle(self.id_, self.title, self.input_message_content)
        c = InlineQueryResultArticle(self.id_, "", self.input_message_content)
        d = InlineQueryResultArticle("", self.title, self.input_message_content)
        e = InlineQueryResultAudio(self.id_, "", "")

        assert a == b
        assert hash(a) == hash(b)
        assert a is not b

        assert a == c
        assert hash(a) == hash(c)

        assert a != d
        assert hash(a) != hash(d)

        assert a != e
        assert hash(a) != hash(e)



#!/usr/bin/env python
#
# A library that provides a Python interface to the Telegram Bot API
# Copyright (C) 2015-2026
# Leandro Toledo de Souza <devs@python-telegram-bot.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser Public License for more details.
#
# You should have received a copy of the GNU Lesser Public License
# along with this program.  If not, see [http://www.gnu.org/licenses/].
"""This module contains an object that represents a Telegram InputFile."""

import mimetypes
from typing import IO
from uuid import uuid4

from telegram._utils.files import guess_file_name, load_file
from telegram._utils.strings import TextEncoding
from telegram._utils.types import FieldTuple

_DEFAULT_MIME_TYPE = "application/octet-stream"


class InputFile:
    """This object represents a Telegram InputFile.

    .. versionchanged:: 20.0

        * The former attribute ``attach`` was renamed to :attr:`attach_name`.
        * Method ``is_image`` was removed. If you pass :obj:`bytes` to :paramref:`obj` and would
          like to have the mime type automatically guessed, please pass :paramref:`filename`
          in addition.

    Args:
        obj (:term:`file object` | :obj:`bytes` | :obj:`str`): An open file descriptor or the files
            content as bytes or string.

            Note:
                If :paramref:`obj` is a string, it will be encoded as bytes via
                :external:obj:`obj.encode('utf-8') <str.encode>`.

            .. versionchanged:: 20.0
                Accept string input.
        filename (:obj:`str`, optional): Filename for this InputFile.
        attach (:obj:`bool`, optional): Pass :obj:`True` if the parameter this file belongs to in
            the request to Telegram should point to the multipart data via an ``attach://`` URI.
            Defaults to `False`.
        read_file_handle (:obj:`bool`, optional): If :obj:`True` and :paramref:`obj` is a file
            handle, the data will be read from the file handle on initialization of this object.
            If :obj:`False`, the file handle will be passed on to the
            :attr:`networking backend <telegram.request.BaseRequest.do_request>` which will have
            to handle the reading. Defaults to :obj:`True`.

            Tip:
                If you upload extremely large files, you may want to set this to :obj:`False` to
                avoid reading the complete file into memory. Additionally, this may be supported
                better by the networking backend (in particular it is handled better by
                the default :class:`~telegram.request.HTTPXRequest`).

            Important:
                If you set this to :obj:`False`, you have to ensure that the file handle is still
                open when the request is made. In particular, the following snippet can *not* work
                as expected.

                .. code-block:: python

                    with open('file.txt', 'rb') as file:
                        input_file = InputFile(file, read_file_handle=False)

                    # here the file handle is already closed and the upload will fail
                    await bot.send_document(chat_id, input_file)

            .. versionadded:: 21.5


    Attributes:
        input_file_content (:obj:`bytes` | :class:`IO`): The binary content of the file to send.
        attach_name (:obj:`str`): Optional. If present, the parameter this file belongs to in
            the request to Telegram should point to the multipart data via a an URI of the form
            ``attach://<attach_name>`` URI.
        filename (:obj:`str`): Filename for the file to be sent.
        mimetype (:obj:`str`): The mimetype inferred from the file to be sent.

    """

    __slots__ = ("attach_name", "filename", "input_file_content", "mimetype")

    def __init__(
        self,
        obj: IO[bytes] | bytes | str,
        filename: str | None = None,
        attach: bool = False,
        read_file_handle: bool = True,
    ):
        if isinstance(obj, bytes):
            self.input_file_content: bytes | IO[bytes] = obj
        elif isinstance(obj, str):
            self.input_file_content = obj.encode(TextEncoding.UTF_8)
        elif read_file_handle:
            reported_filename, self.input_file_content = load_file(obj)
            filename = filename or reported_filename
        else:
            self.input_file_content = obj
            filename = filename or guess_file_name(obj)

        self.attach_name: str | None = "attached" + uuid4().hex if attach else None

        if filename:
            self.mimetype: str = (
                mimetypes.guess_type(filename, strict=False)[0] or _DEFAULT_MIME_TYPE
            )
        else:
            self.mimetype = _DEFAULT_MIME_TYPE

        self.filename: str = filename or self.mimetype.replace("/", ".")

    @property
    def field_tuple(self) -> FieldTuple:
        """Field tuple representing the contents of the file for upload to the Telegram servers.

        .. versionchanged:: 21.5
            Content may now be a file handle.

        Returns:
            tuple[:obj:`str`, :obj:`bytes` | :class:`IO`, :obj:`str`]:
        """
        return self.filename, self.input_file_content, self.mimetype

    @property
    def attach_uri(self) -> str | None:
        """URI to insert into the JSON data for uploading the file. Returns :obj:`None`, if
        :attr:`attach_name` is :obj:`None`.
        """
        return f"attach://{self.attach_name}" if self.attach_name else None



#!/usr/bin/env python
#
# A library that provides a Python interface to the Telegram Bot API
# Copyright (C) 2015-2026
# Leandro Toledo de Souza <devs@python-telegram-bot.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser Public License for more details.
#
# You should have received a copy of the GNU Lesser Public License
# along with this program.  If not, see [http://www.gnu.org/licenses/].
"""This module contains objects related to Telegram video chats."""

import datetime as dtm
from collections.abc import Sequence
from typing import TYPE_CHECKING

from telegram._telegramobject import TelegramObject
from telegram._user import User
from telegram._utils.argumentparsing import parse_sequence_arg, to_timedelta
from telegram._utils.datetime import (
    extract_tzinfo_from_defaults,
    from_timestamp,
    get_timedelta_value,
)
from telegram._utils.types import JSONDict, TimePeriod

if TYPE_CHECKING:
    from telegram import Bot


class VideoChatStarted(TelegramObject):
    """
    This object represents a service message about a video
    chat started in the chat. Currently holds no information.

    .. versionadded:: 13.4
    .. versionchanged:: 20.0
        This class was renamed from ``VoiceChatStarted`` in accordance to Bot API 6.0.
    """

    __slots__ = ()

    def __init__(self, *, api_kwargs: JSONDict | None = None) -> None:
        super().__init__(api_kwargs=api_kwargs)

        self._freeze()


class VideoChatEnded(TelegramObject):
    """
    This object represents a service message about a
    video chat ended in the chat.

    Objects of this class are comparable in terms of equality.
    Two objects of this class are considered equal, if their
    :attr:`duration` are equal.

    .. versionadded:: 13.4
    .. versionchanged:: 20.0
        This class was renamed from ``VoiceChatEnded`` in accordance to Bot API 6.0.

    .. versionchanged:: v22.2
       As part of the migration to representing time periods using ``datetime.timedelta``,
       equality comparison now considers integer durations and equivalent timedeltas as equal.

    Args:
        duration (:obj:`int` | :class:`datetime.timedelta`): Voice chat duration
            in seconds.

            .. versionchanged:: v22.2
                |time-period-input|

    Attributes:
        duration (:obj:`int` | :class:`datetime.timedelta`): Voice chat duration in seconds.

            .. deprecated:: v22.2
                |time-period-int-deprecated|

    """

    __slots__ = ("_duration",)

    def __init__(
        self,
        duration: TimePeriod,
        *,
        api_kwargs: JSONDict | None = None,
    ) -> None:
        super().__init__(api_kwargs=api_kwargs)
        self._duration: dtm.timedelta = to_timedelta(duration)
        self._id_attrs = (self._duration,)

        self._freeze()

    @property
    def duration(self) -> int | dtm.timedelta:
        return get_timedelta_value(  # type: ignore[return-value]
            self._duration, attribute="duration"
        )


class VideoChatParticipantsInvited(TelegramObject):
    """
    This object represents a service message about new members invited to a video chat.

    Objects of this class are comparable in terms of equality.
    Two objects of this class are considered equal, if their :attr:`users` are equal.

    .. versionadded:: 13.4
    .. versionchanged:: 20.0
        This class was renamed from ``VoiceChatParticipantsInvited`` in accordance to Bot API 6.0.

    Args:
        users (Sequence[:class:`telegram.User`]): New members that were invited to the video chat.

            .. versionchanged:: 20.0
                |sequenceclassargs|

    Attributes:
        users (tuple[:class:`telegram.User`]): New members that were invited to the video chat.

            .. versionchanged:: 20.0
                |tupleclassattrs|

    """

    __slots__ = ("users",)

    def __init__(
        self,
        users: Sequence[User],
        *,
        api_kwargs: JSONDict | None = None,
    ) -> None:
        super().__init__(api_kwargs=api_kwargs)
        self.users: tuple[User, ...] = parse_sequence_arg(users)
        self._id_attrs = (self.users,)

        self._freeze()

    @classmethod
    def de_json(cls, data: JSONDict, bot: "Bot | None" = None) -> "VideoChatParticipantsInvited":
        """See :meth:`telegram.TelegramObject.de_json`."""
        data = cls._parse_data(data)

        data["users"] = User.de_list(data.get("users", []), bot)
        return super().de_json(data=data, bot=bot)


class VideoChatScheduled(TelegramObject):
    """This object represents a service message about a video chat scheduled in the chat.

    Objects of this class are comparable in terms of equality. Two objects of this class are
    considered equal, if their :attr:`start_date` are equal.

    .. versionchanged:: 20.0
        This class was renamed from ``VoiceChatScheduled`` in accordance to Bot API 6.0.

    Args:
        start_date (:obj:`datetime.datetime`): Point in time (Unix timestamp) when the video
            chat is supposed to be started by a chat administrator

            .. versionchanged:: 20.3
                |datetime_localization|
    Attributes:
        start_date (:obj:`datetime.datetime`): Point in time (Unix timestamp) when the video
            chat is supposed to be started by a chat administrator

            .. versionchanged:: 20.3
                |datetime_localization|

    """

    __slots__ = ("start_date",)

    def __init__(
        self,
        start_date: dtm.datetime,
        *,
        api_kwargs: JSONDict | None = None,
    ) -> None:
        super().__init__(api_kwargs=api_kwargs)
        self.start_date: dtm.datetime = start_date

        self._id_attrs = (self.start_date,)

        self._freeze()

    @classmethod
    def de_json(cls, data: JSONDict, bot: "Bot | None" = None) -> "VideoChatScheduled":
        """See :meth:`telegram.TelegramObject.de_json`."""
        data = cls._parse_data(data)

        # Get the local timezone from the bot if it has defaults
        loc_tzinfo = extract_tzinfo_from_defaults(bot)

        data["start_date"] = from_timestamp(data.get("start_date"), tzinfo=loc_tzinfo)

        return super().de_json(data=data, bot=bot)



#!/usr/bin/env python
#
# A library that provides a Python interface to the Telegram Bot API
# Copyright (C) 2015-2026
# Leandro Toledo de Souza <devs@python-telegram-bot.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser Public License for more details.
#
# You should have received a copy of the GNU Lesser Public License
# along with this program.  If not, see [http://www.gnu.org/licenses/].
"""This module contains functions which confirm that the parameters of our methods and classes
match the official API. It also checks if the type annotations are correct and if the parameters
are required or not."""

import datetime as dtm
import inspect
import logging
import re
from collections.abc import Sequence
from types import FunctionType
from typing import Any

from telegram._utils.defaultvalue import DefaultValue
from telegram._utils.types import FileInput, ODVInput
from telegram.ext import Defaults
from tests.test_official.exceptions import ParamTypeCheckingExceptions as PTCE
from tests.test_official.exceptions import ignored_param_requirements
from tests.test_official.helpers import (
    _extract_words,
    _get_params_base,
    _unionizer,
    cached_type_hints,
    extract_mappings,
    resolve_forward_refs_in_type,
    wrap_with_none,
)
from tests.test_official.scraper import TelegramParameter

ARRAY_OF_PATTERN = r"Array of(?: Array of)? ([\w\,\s]*)"

# In order to evaluate the type annotation, we need to first have a mapping of the types
# specified in the official API to our types. The keys are types in the column of official API.
TYPE_MAPPING: dict[str, set[Any]] = {
    "Integer or String": {int | str},
    "Integer": {int},
    "String": {str},
    r"Boolean|True": {bool},
    r"Float(?: number)?": {float},
    # Distinguishing 1D and 2D Sequences and finding the inner type is done later.
    ARRAY_OF_PATTERN: {Sequence},
    r"InputFile(?: or String)?": {resolve_forward_refs_in_type(FileInput)},
}

ALL_DEFAULTS = inspect.getmembers(Defaults, lambda x: isinstance(x, property))

DATETIME_REGEX = re.compile(
    r"""([_]+|\b)  # check for word boundary or underscore
    date       # check for "date"
    [^\w]*\b   # optionally check for a word after 'date'
    """,
    re.VERBOSE,
)
TIMEDELTA_REGEX = re.compile(r"((in|number of) seconds)|(\w+_period$)")

log = logging.debug


def check_required_param(
    tg_param: TelegramParameter, param: inspect.Parameter, method_or_obj_name: str
) -> bool:
    """Checks if the method/class parameter is a required/optional param as per Telegram docs.

    Returns:
        :obj:`bool`: The boolean returned represents whether our parameter's requirement (optional
            or required) is the same as Telegram's or not.
    """
    is_ours_required = param.default is inspect.Parameter.empty
    # Handle cases where we provide convenience intentionally-
    if param.name in ignored_param_requirements(method_or_obj_name):
        return True
    return tg_param.param_required is is_ours_required


def check_defaults_type(ptb_param: inspect.Parameter) -> bool:
    return DefaultValue.get_value(ptb_param.default) is None


def check_param_type(
    ptb_param: inspect.Parameter,
    tg_parameter: TelegramParameter,
    obj: FunctionType | type,
) -> tuple[bool, type]:
    """This function checks whether the type annotation of the parameter is the same as the one
    specified in the official API. It also checks for some special cases where we accept more types

    Args:
        ptb_param: The parameter object from our methods/classes
        tg_parameter: The table row corresponding to the parameter from official API.
        obj: The object (method/class) that we are checking.

    Returns:
        :obj:`tuple`: A tuple containing:
            * :obj:`bool`: The boolean returned represents whether our parameter's type annotation
              is the same as Telegram's or not.
            * :obj:`type`: The expected type annotation of the parameter.
    """
    # PRE-PROCESSING:
    tg_param_type: str = tg_parameter.param_type
    is_class = inspect.isclass(obj)
    ptb_annotation = cached_type_hints(obj, is_class).get(ptb_param.name)

    # Let's check for a match:
    # In order to evaluate the type annotation, we need to first have a mapping of the types
    # (see TYPE_MAPPING comment defined at the top level of this module)
    mapped: set[type] = _get_params_base(tg_param_type, TYPE_MAPPING)

    # We should have a maximum of one match.
    assert len(mapped) <= 1, f"More than one match found for {tg_param_type}"

    # it may be a list of objects, so let's extract them using _extract_words:
    org_mapped_type = _unionizer(_extract_words(tg_param_type)) if not mapped else mapped.pop()
    # If the parameter is not required by TG, `None` should be added to `mapped_type`
    mapped_type = wrap_with_none(tg_parameter, org_mapped_type, obj)

    log(
        "At the end of PRE-PROCESSING, the values of variables are:\n"
        "Parameter name: %s\n"
        "ptb_annotation= %s\n"
        "mapped_type= %s\n"
        "tg_param_type= %s\n"
        "tg_parameter.param_required= %s\n",
        ptb_param.name,
        ptb_annotation,
        mapped_type,
        tg_param_type,
        tg_parameter.param_required,
    )

    # CHECKING:
    # Each branch manipulates the `mapped_type` (except for 5) ) to match the `ptb_annotation`.

    # 1) HANDLING ARRAY TYPES:
    # Now let's do the checking, starting with "Array of ..." types.
    if "Array of " in tg_param_type:
        # For exceptions just check if they contain the annotation
        if any(ptb_param.name in key for key in PTCE.ARRAY_OF_EXCEPTIONS):
            for (p_name, is_expected_class), exception_type in PTCE.ARRAY_OF_EXCEPTIONS.items():
                if ptb_param.name == p_name and is_class is is_expected_class:
                    log("Checking that `%s` is an exception!\n", ptb_param.name)
                    return exception_type in str(ptb_annotation), Sequence

        obj_match: re.Match | None = re.search(ARRAY_OF_PATTERN, tg_param_type)
        if obj_match is None:
            raise AssertionError(f"Array of {tg_param_type} not found in {ptb_param.name}")
        obj_str: str = obj_match.group(1)
        # is obj a regular type like str?
        array_map: set[type] = _get_params_base(obj_str, TYPE_MAPPING)

        mapped_type = _unionizer(_extract_words(obj_str)) if not array_map else array_map.pop()

        if "Array of Array of" in tg_param_type:
            log("Array of Array of type found in `%s`\n", tg_param_type)
            mapped_type = Sequence[Sequence[mapped_type]]
        else:
            log("Array of type found in `%s`\n", tg_param_type)
            mapped_type = Sequence[mapped_type]

    # 2) HANDLING OTHER TYPES:
    # Special case for send_* methods where we accept more types than the official API:
    elif additional_types := extract_mappings(PTCE.ADDITIONAL_TYPES, obj, ptb_param.name):
        log("Checking that `%s` accepts additional types for some parameters!\n", obj.__name__)
        for at in additional_types:
            log("Checking that `%s` is an additional type for `%s`!\n", at, ptb_param.name)
            mapped_type = mapped_type | at

    # 3) HANDLING DATETIMES:
    elif (
        re.search(
            DATETIME_REGEX,
            ptb_param.name,
        )
        or "Unix time" in tg_parameter.param_description
    ):
        log("Checking that `%s` is a datetime!\n", ptb_param.name)
        # If it's a class, we only accept datetime as the parameter
        mapped_type = dtm.datetime if is_class else mapped_type | dtm.datetime

    # 4) HANDLING TIMEDELTA:
    elif re.search(TIMEDELTA_REGEX, tg_parameter.param_description) or re.search(
        TIMEDELTA_REGEX, ptb_param.name
    ):
        log("Checking that `%s` is a timedelta!\n", ptb_param.name)
        mapped_type = mapped_type | dtm.timedelta

    # 5) COMPLEX TYPES:
    # Some types are too complicated, so we replace our annotation with a simpler type:
    elif overrides := extract_mappings(PTCE.COMPLEX_TYPES, obj, ptb_param.name):
        exception_type = overrides[0]
        log("Converting `%s` to a simpler type!\n", ptb_param.name)
        ptb_annotation = wrap_with_none(tg_parameter, exception_type, obj)

    # 6) HANDLING DEFAULTS PARAMETERS:
    # Classes whose parameters are all ODVInput should be converted and checked.
    elif obj.__name__ in PTCE.IGNORED_DEFAULTS_CLASSES:
        log("Checking that `%s`'s param is ODVInput:\n", obj.__name__)
        # We have to use org_mapped_type here, because ODVInput will not take a None value as well
        mapped_type = ODVInput[org_mapped_type]
    elif not (
        # Defaults checking should not be done for:
        # 1. Parameters that have name conflict with `Defaults.name`
        is_class
        and obj.__name__ in ("ReplyParameters", "Message", "ExternalReplyInfo")
        and ptb_param.name in PTCE.IGNORED_DEFAULTS_PARAM_NAMES
    ):
        # Now let's check if the parameter is a Defaults parameter, it should be
        for name, _ in ALL_DEFAULTS:
            if name == ptb_param.name or "parse_mode" in ptb_param.name:
                log("Checking that `%s` is a Defaults parameter!\n", ptb_param.name)
                # We have to use org_mapped_type here, because ODVInput will not take a None value
                mapped_type = ODVInput[org_mapped_type]
                break

    # RESULTS:-
    mapped_type = wrap_with_none(tg_parameter, mapped_type, obj)
    mapped_type = resolve_forward_refs_in_type(mapped_type)
    log(
        "At RESULTS, we are comparing:\nptb_annotation= %s\nmapped_type= %s\n",
        ptb_annotation,
        mapped_type,
    )
    return mapped_type == ptb_annotation, mapped_type
